//KCInterface.h
#pragma once
// Interface declarations for native control to be redistributed with KC installation
#include "DITypes.h"
#include "Misc.h"
#include "TIds.h"

class IKO_Geom_BSplineSurface;

#define DI_AMBIENT    0
#define DI_DIFFUSE    1
#define DI_SPECULAR   2
#define DI_SHININESS  3

// Render Type
#define RENDER_TYPE_NORENDER	0
#define RENDER_TYPE_SOLID		1
#define RENDER_TYPE_WIREFRAME	2

#define SYMM_TYPE_NOSYMM		0
#define SYMM_TYPE_AXIAL			1
#define SYMM_TYPE_MIRROR		2
#define SYMM_TYPE_TRANSLATION	3

#define OBJECT_TYPE_NOOBJECT	0
#define OBJECT_TYPE_3DOBJECT	1
#define OBJECT_TYPE_SOR			2

											// Decimal
#define DI_MOVE_PAN				0x0001		// 1
#define DI_MOVE_ZOOM			0x0001<<1	// 2
#define DI_MOVE_ROTATE			0x0001<<2	// 4

#define DI_HIGHLIGHT_POINT		0x0001<<7	// 128
#define DI_HIGHLIGHT_LINE		0x0001<<8	// 256
#define DI_HIGHLIGHT_SURFACE	0x0001<<9	// 512
#define DI_TRANSPARENCY			0x0001<<10	// 1024
#define DI_PROJECT_PERSPECTIVE	0x0001<<11	// 2048

#define DI_VIEW_MODE_VIEW		1
#define DI_VIEW_MODE_MODIFY		2

#ifndef _NET
// {44886EA6-F4FC-420a-A27B-BF095946D608}
static const GUID CLSID_Sect3DGenerator = { 0x44886ea6, 0xf4fc, 0x420a, { 0xa2, 0x7b, 0xbf, 0x9, 0x59, 0x46, 0xd6, 0x8 } };
#endif
#define PROGID_SECT_3D_GENERATOR  "KERNELCAD.Section3D.1"

#ifndef _NET
// {BF14B870-93AF-4822-89F3-A78ED9207483}
static const GUID CLSID_SectSORGenerator = { 0xbf14b870, 0x93af, 0x4822, { 0x89, 0xf3, 0xa7, 0x8e, 0xd9, 0x20, 0x74, 0x83 } };
#endif	//_NET
#define PROGID_SECT_SOR_GENERATOR  "KERNELCAD.SectionSOR.1"

#ifndef _NET
// {E70C92E2-0F1B-4d3c-8DDB-C50374604FB6}
static const GUID CLSID_Model = { 0xe70c92e2, 0xf1b, 0x4d3c, { 0x8d, 0xdb, 0xc5, 0x3, 0x74, 0x60, 0x4f, 0xb6 } };
#endif
#define PROGID_MODEL  "KERNELCAD.Model.1"

/////////////////////////////////// DIObjGenerator ///////////////////////////////////////
#ifndef _NET
// {CBFE8FE5-5873-48b1-8F10-678D818E218E}
static const GUID CLSID_DIObjGenerator = { 0xcbfe8fe5, 0x5873, 0x48b1, { 0x8f, 0x10, 0x67, 0x8d, 0x81, 0x8e, 0x21, 0x8e} };
#endif
#define PROGID_DIOBJGENERATOR  "KERNELCAD.DIObjGenerator.1"

// {DA3EF301-E066-42EE-BCF4-4EAFCB41DCB5}
static const GUID CLSID_KCGlobAPI = { 0xda3ef301, 0xe066, 0x42ee, { 0xbc, 0xf4, 0x4e, 0xaf, 0xcb, 0x41, 0xdc, 0xb5} };
#define PROGID_KCGLOBAPI  "KERNELCAD.KCAPI.1"

// {4DC98E26-A609-4861-85C5-31E7EA4823C5}
static const GUID IID_IDictionary_KC = { 0x4dc98e26, 0xa609, 0x4861, { 0x85, 0xc5, 0x31, 0xe7, 0xea, 0x48, 0x23, 0xc5 } };
class IDictionary_KC : public IUnknown
{
public:
	ICALLTYPE GetParamCount(int type, int* count)=0;
	ICALLTYPE GetParamName(int id, BSTR *name)=0;
	ICALLTYPE GetParamDescription(char* name, BSTR *description)=0;
	ICALLTYPE GetParamType(char* name, int* type)=0;
	ICALLTYPE GetInt(char* name, int *val)=0;
	ICALLTYPE SetInt(char* name, int val)=0;
	ICALLTYPE GetDouble(char* name, double *val)=0;
	ICALLTYPE SetDouble(char* name, double val)=0;
	ICALLTYPE GetBool(char* name, VARIANT_BOOL *val)=0;
	ICALLTYPE SetBool(char* name, VARIANT_BOOL val)=0;
	ICALLTYPE Get3State(char* name, KC3State *val)=0;
	ICALLTYPE Set3State(char* name, KC3State val)=0;
	ICALLTYPE GetString(char* name, BSTR *val)=0;
	ICALLTYPE SetString(char* name, char *val)=0;
	ICALLTYPE Get(char* name, VARIANT *param)=0;
	ICALLTYPE Set(char* name, VARIANT *param)=0;
	ICALLTYPE GetInterface(char* name, IUnknown** interf)=0;
	ICALLTYPE SetInterface(char* name, IUnknown* interf)=0;
	IUNKNOWN_
};

#ifndef _NET
// {39C6B29E-EC14-4b88-AA47-F46E3D5D2B1A}
static const GUID IID_IElement = { 0x39c6b29e, 0xec14, 0x4b88, { 0xaa, 0x47, 0xf4, 0x6e, 0x3d, 0x5d, 0x2b, 0x1a } };
class IElement : public IUnknown
{
public:
	ICALLTYPE GetParamCount(int *plCount)=0;
	ICALLTYPE GetParam( int id, double *pd)=0;
	ICALLTYPE SetParam( int id, double d)=0;
	ICALLTYPE GetNamedParam(char* szParamName, double *pd)=0;
	ICALLTYPE SetNamedParam(char* szParamName, double d)=0;
	ICALLTYPE GetParamName( int id, char*  szParamName)=0;
	ICALLTYPE GetParamId(char*  szParamName, int *plId)=0;
	ICALLTYPE GetElemClassName(char*  szName)=0;
	IUNKNOWN_
};
#endif

// {BD5DCE13-A8D6-489e-A249-2797DAC28184}
static const GUID IID_IElementStd = { 0xbd5dce13, 0xa8d6, 0x489e, { 0xa2, 0x49, 0x27, 0x97, 0xda, 0xc2, 0x81, 0x84 } };
class IElementStd : public IUnknown
{
public:
	ICALLTYPE GetParamCount(int *plCount)=0;
	ICALLTYPE GetParam( int id, double *pd)=0;
	ICALLTYPE SetParam( int id, double d)=0;
	ICALLTYPE GetNamedParam(char* szParamName, double *pd)=0;
	ICALLTYPE SetNamedParam(char* szParamName, double d)=0;
	ICALLTYPE GetParamName( int id, char*  szParamName)=0;
	ICALLTYPE GetParamId(char*  szParamName, int *plId)=0;
	ICALLTYPE GetElemClassName(char*  szName)=0;
	IUNKNOWN_
};

#ifndef _NET
// {FC85304C-CDA7-4e39-86DD-EDFF386C6F52}
static const GUID IID_IElem = { 0xfc85304c, 0xcda7, 0x4e39, { 0x86, 0xdd, 0xed, 0xff, 0x38, 0x6c, 0x6f, 0x52 } };
class IElem : public IUnknown
{
public:
	ICALLTYPE GetParamCount(int *plCount)=0;
	ICALLTYPE GetParam( int id, double *pd)=0;
	ICALLTYPE SetParam( int id, double d, int* result)=0;
	ICALLTYPE GetNamedParam(char* szParamName, double *pd)=0;
	ICALLTYPE SetNamedParam(char* szParamName, double d, int* result)=0;
	ICALLTYPE GetParamName( int id, char*  szParamName)=0;
	ICALLTYPE GetParamId(char*  szParamName, int *plId)=0;
	ICALLTYPE GetElemClassName(char*  szName)=0;
	IUNKNOWN_
};
#endif

// {9FA0B14A-1903-4196-8119-4034188BA5AD}
static const GUID IID_IElemStd = { 0x9fa0b14a, 0x1903, 0x4196, { 0x81, 0x19, 0x40, 0x34, 0x18, 0x8b, 0xa5, 0xad } };
class IElemStd : public IUnknown
{
public:
	ICALLTYPE GetParamCount(int *plCount)=0;
	ICALLTYPE GetParam( int id, double *pd)=0;
	ICALLTYPE SetParam( int id, double d, int* result)=0;
	ICALLTYPE GetNamedParam(char* szParamName, double *pd)=0;
	ICALLTYPE SetNamedParam(char* szParamName, double d, int* result)=0;
	ICALLTYPE GetParamName( int id, char*  szParamName)=0;
	ICALLTYPE GetParamId(char*  szParamName, int *plId)=0;
	ICALLTYPE GetElemClassName(char*  szName)=0;
	IUNKNOWN_
};

// {BB6647C8-2C13-4720-B92B-5A1A75343389}
static const GUID IID_IRangeParam = { 0xbb6647c8, 0x2c13, 0x4720, { 0xb9, 0x2b, 0x5a, 0x1a, 0x75, 0x34, 0x33, 0x89 } };
class IRangeParam : public IUnknown
{
public:
	ICALLTYPE GetRange( int nParamID, double *pdMin, double *pdMax)=0;
	ICALLTYPE GetRangeCount( int nParamID, int *plCount)=0;
	ICALLTYPE GetRange( int nParamID, int nRange, double *pdMin, double *pdMax)=0;
	IUNKNOWN_
};

// {2D2C0BC3-28FE-4886-8591-1426C3CA2098}
static const GUID IID_IStrip = { 0x2d2c0bc3, 0x28fe, 0x4886, { 0x85, 0x91, 0x14, 0x26, 0xc3, 0xca, 0x20, 0x98 } };
class IStrip : public IUnknown
{
public:

	ICALLTYPE GetJointCount(int * pnCount)=0;
	ICALLTYPE GetElement( int index, VARIANT_BOOL segment, IElement **ppIElem)=0;
	ICALLTYPE  IsClosed(VARIANT_BOOL *pbClosed)=0;
	IUNKNOWN_
};

// {F87D6FED-291D-44f3-8E34-FBE71B83C6E2}
static const GUID IID_IProfiles = { 0xf87d6fed, 0x291d, 0x44f3, { 0x8e, 0x34, 0xfb, 0xe7, 0x1b, 0x83, 0xc6, 0xe2 } };
class IProfiles : public IUnknown
{
public:
	ICALLTYPE GetProfile( int id, IStrip **ppIStrip)=0;
	ICALLTYPE GetNamedProfile(char*  szParamName, IStrip **ppIStrip)=0;
	IUNKNOWN_
};

// {BBF3C596-9FE4-4d51-B711-7C47C1F582A5}
static const GUID IID_IAxiBase = { 0xbbf3c596, 0x9fe4, 0x4d51, { 0xb7, 0x11, 0x7c, 0x47, 0xc1, 0xf5, 0x82, 0xa5 } };
class IAxiBase : public IUnknown
{
public:
	ICALLTYPE GetCrossPosition(double *pz)=0;
	ICALLTYPE SetCrossPosition(double z)=0;
	ICALLTYPE GetCount(int *pnCount)=0;
	ICALLTYPE GetKnotValue( int knot, double *pz)=0;
	ICALLTYPE SetKnotValue( int knot, double z)=0;
	IUNKNOWN_
};

// {EBCB835B-CF9C-474f-BCCE-B360A69DCD1D}
static const GUID IID_IProfiledElement_KC = { 0xebcb835b, 0xcf9c, 0x474f, { 0xbc, 0xce, 0xb3, 0x60, 0xa6, 0x9d, 0xcd, 0x1d } };
class IProfiledElement_KC : public IUnknown
{
public:
	ICALLTYPE GetCrossection(double position, IElement** crossection) = 0;
	IUNKNOWN_
};

// {2DF73949-1E89-4f5c-B303-ABCE524154E8}
static const GUID IID_ISectTopology = { 0x2df73949, 0x1e89, 0x4f5c, { 0xb3, 0x3, 0xab, 0xce, 0x52, 0x41, 0x54, 0xe8 } };
class ISectTopology : public IUnknown
{
public:
	ICALLTYPE InsertNewKnot(double z)=0;
	ICALLTYPE BreakSurfaceElem( int nSurfElem )=0;
	IUNKNOWN_
};

// {85E08141-728D-4c19-B648-AFCD5EE1EFF1}
static const GUID IID_I3DObject = { 0x85e08141, 0x728d, 0x4c19, { 0xb6, 0x48, 0xaf, 0xcd, 0x5e, 0xe1, 0xef, 0xf1 } };
class I3DObject : public IUnknown
{
public:
	ICALLTYPE GetOrigin(double *x, double *y, double *z)=0;
	ICALLTYPE SetOrigin(double x, double y, double z)=0;
	ICALLTYPE GetAxis(int nAxis, double *x, double *y, double *z)=0;
	ICALLTYPE Translate(double vx, double vy, double vz)=0;
	ICALLTYPE Rotate(double dAng, double xAxis, double yAxis, double zAxis, 
						double vxDirAxis, double vyDirAxis, double vzDirAxis)=0;
	IUNKNOWN_
};

// {862940CE-1F49-4f79-A82B-B532A80F864B}
static const GUID	IID_IFrame = { 0x862940ce, 0x1f49, 0x4f79, { 0xa8, 0x2b, 0xb5, 0x32, 0xa8, 0xf, 0x86, 0x4b } };
class IFrame : public I3DObject
{
public:
	ICALLTYPE RotateStd(double dAng, int nAxis) = 0;
	ICALLTYPE SetAxes(int nAxis0, DIVect* axis0, int nAxis1, DIVect* axis1, bool bOrientation) = 0;
	ICALLTYPE Reset() = 0;
	ICALLTYPE ResetOrientation() = 0;
	IUNKNOWN_
};

// {4EBFBEF0-E222-4b74-BE72-26A148BD8190}
static const GUID IID_IFrame2 = { 0x4ebfbef0, 0xe222, 0x4b74, { 0xbe, 0x72, 0x26, 0xa1, 0x48, 0xbd, 0x81, 0x90 } };
class IFrame2 : public IUnknown
{
public:
	ICALLTYPE Translate(double vx, double vy, double vz) = 0;
	ICALLTYPE Rotate(double dAng, double vxDirAxis, double vyDirAxis, double vzDirAxis) = 0;
	ICALLTYPE RotateEx(double dAng, double xAxis, double yAxis, double zAxis, double vxDirAxis, double vyDirAxis, double vzDirAxis) = 0;
	ICALLTYPE Copy(IFrame* frame) = 0;
	ICALLTYPE SetupAxes(int nAxis0, DIVect* axis, int nAxis1) = 0;
	IUNKNOWN_
};

// {4D1F2EA2-897D-4c9c-8375-DD9046D300E4}
static const GUID IID_IFrameEx = { 0x4d1f2ea2, 0x897d, 0x4c9c, { 0x83, 0x75, 0xdd, 0x90, 0x46, 0xd3, 0x0, 0xe4 } };
class IFrameEx : public IUnknown
{
public:
	ICALLTYPE ToLocal(DIPoint* point) = 0;
	ICALLTYPE ToGlobal(DIPoint* point) = 0;
	ICALLTYPE ToLocalVector(DIVect* vect) = 0;
	ICALLTYPE ToGlobalVector(DIVect* vect) = 0;
	IUNKNOWN_
};

// {D0C32852-3B88-4e15-AF33-5429023CBF8C}
static const GUID IID_IFrame3 = { 0xd0c32852, 0x3b88, 0x4e15, { 0xaf, 0x33, 0x54, 0x29, 0x2, 0x3c, 0xbf, 0x8c } };
class IFrame3 : public IUnknown
{
public:
	ICALLTYPE GetOrigin(double *x, double *y, double *z) = 0;
	ICALLTYPE SetOrigin(double x, double y, double z) = 0;
	ICALLTYPE GetAxis(int axis, double *vx, double *vy, double *vz) = 0;
	ICALLTYPE Translate(double vx, double vy, double vz) = 0;
	ICALLTYPE TranslateGlobal(double vx, double vy, double vz) = 0;
	ICALLTYPE Rotate(double angle, double xAxis, double yAxis, double zAxis, double vxDirAxis, double vyDirAxis, double vzDirAxis) = 0;
	ICALLTYPE RotateGlobal(double angle, double xAxis, double yAxis, double zAxis, double vxDirAxis, double vyDirAxis, double vzDirAxis) = 0;
	ICALLTYPE SetAxes(int axisInd0, DIVect* axis0, int axisInd1, DIVect* axis1) = 0;
	ICALLTYPE SetupAxes(int axis0, DIVect* axis, int axis1) = 0;
	ICALLTYPE Reset() = 0;
	ICALLTYPE ResetOrientation() = 0;
	ICALLTYPE Copy(IUnknown* frame) = 0;
	ICALLTYPE ToLocal(double *x, double *y, double *z) = 0;
	ICALLTYPE ToGlobal(double *x, double *y, double *z) = 0;
	ICALLTYPE Push() = 0;
	ICALLTYPE Pop() = 0;
	ICALLTYPE ToLocalVector(DIVect* vect) = 0;
	ICALLTYPE ToGlobalVector(DIVect* vect) = 0;
	ICALLTYPE ToLocalFrame(IFrame3* frame) = 0;
	ICALLTYPE ToGlobalFrame(IFrame3* frame) = 0;
	ICALLTYPE Invert() = 0;
	IUNKNOWN_
};

// {5D6F3BDD-1D74-4795-ADA1-B46EEE1C29A4}
static const GUID IID_IObjectOriented = { 0x5d6f3bdd, 0x1d74, 0x4795, { 0xad, 0xa1, 0xb4, 0x6e, 0xee, 0x1c, 0x29, 0xa4 } };
class IObjectOriented : public IUnknown
{
public:
	ICALLTYPE GetFrame(IFrame3** frame) = 0;
	IUNKNOWN_
};

// {0194F64C-EFEB-4970-925E-20E8FEABFB61}
static const GUID IID_IItem = { 0x194f64c, 0xefeb, 0x4970, { 0x92, 0x5e, 0x20, 0xe8, 0xfe, 0xab, 0xfb, 0x61 } };
class IItem : public IUnknown
{
public:
	ICALLTYPE GetVisibleFlag(VARIANT_BOOL* visible)=0;
	ICALLTYPE SetVisible(VARIANT_BOOL visible)=0;
	ICALLTYPE GetName(BSTR* name)=0;
	ICALLTYPE SetName(char* name)=0;
	ICALLTYPE GetParent(IItem **parent)=0;
	ICALLTYPE IsSelected(VARIANT_BOOL* selected)=0;
	ICALLTYPE SetModified()=0;
	IUNKNOWN_
};

// {A66D0B80-9A5A-4fce-835F-F3E0619DE947}
static const GUID IID_ISection = { 0xa66d0b80, 0x9a5a, 0x4fce, { 0x83, 0x5f, 0xf3, 0xe0, 0x61, 0x9d, 0xe9, 0x47 } };
class ISection : public IUnknown
{
public:
#ifdef _NET
	ICALLTYPE GetSectionType(BSTR *pbstrType)=0;
	ICALLTYPE GetName(BSTR *pbstrName)=0;
#else
	ICALLTYPE GetSectionType(char*  szType)=0;
	ICALLTYPE GetName(char*  szName)=0;
#endif
	ICALLTYPE SetName(char*  szName)=0;
	ICALLTYPE GetGeomResolution(int *pnRes)=0;
	ICALLTYPE SetGeomResolution(int nMinSimplexPerDim)=0;
	ICALLTYPE GetChildCount(int* pnCount)=0;
	ICALLTYPE GetChild(int index, ISection **ppISection)=0;
	ICALLTYPE GetVisibleFlag(VARIANT_BOOL *pbVisible)=0;
	ICALLTYPE SetVisible(VARIANT_BOOL bVisible)=0;
	ICALLTYPE GetParent(ISection **ppISection)=0;
	ICALLTYPE GetItemCount(int* count)=0;
	ICALLTYPE GetItem(int index, IItem **iItem)=0;
	IUNKNOWN_
};

// {B2EB9715-84C4-46b2-B334-70DB373A0234}	Unpublished
static const GUID IID_ISectionStd = { 0xb2eb9715, 0x84c4, 0x46b2, { 0xb3, 0x34, 0x70, 0xdb, 0x37, 0x3a, 0x2, 0x34 } };
class ISectionStd : public IUnknown
{
public:
	ICALLTYPE GetSectionType(char*  szType)=0;
	ICALLTYPE GetName(char*  szName)=0;
	ICALLTYPE SetName(char*  szName)=0;
	ICALLTYPE GetGeomResolution(int *pnRes)=0;
	ICALLTYPE SetGeomResolution(int nMinSimplexPerDim)=0;
	ICALLTYPE GetChildCount(int* pnCount)=0;
	ICALLTYPE GetChild(int index, ISection **ppISection)=0;
	ICALLTYPE GetVisibleFlag(VARIANT_BOOL *pbVisible)=0;
	ICALLTYPE SetVisible(VARIANT_BOOL bVisible)=0;
	ICALLTYPE GetParent(ISection **ppISection)=0;
	ICALLTYPE GetItemCount(int* count)=0;
	ICALLTYPE GetItem(int index, IItem **iItem)=0;
	IUNKNOWN_
};

// {6648CD85-98E6-41d2-B898-018098488C31}
static const GUID IID_ISection2 = { 0x6648cd85, 0x98e6, 0x41d2, { 0xb8, 0x98, 0x1, 0x80, 0x98, 0x48, 0x8c, 0x31 } };
class ISection2 : public IUnknown
{
public:
	ICALLTYPE GetFrameVisibleFlag(VARIANT_BOOL *pbVisible)=0;
	ICALLTYPE SetFrameVisible(VARIANT_BOOL bVisible)=0;
	ICALLTYPE Show(VARIANT_BOOL show, VARIANT_BOOL includeChildren, VARIANT_BOOL childrenOnly)=0;
	ICALLTYPE SetColor(float fRed, float fGreen, float fBlue)=0;
	ICALLTYPE Detach()=0;
	ICALLTYPE RemoveAll() = 0;
	ICALLTYPE RemoveChild(IItem *item) = 0;
	ICALLTYPE Relocate(IFrame* newLocationOrientation) = 0;
	ICALLTYPE SaveAs(char* filePath, IDictionary_KC* options, VARIANT_BOOL* success)=0;
	IUNKNOWN_
};

// {B2999327-7E17-423d-9BC5-127D9965D9AF}
static const GUID IID_IBoundary = { 0xb2999327, 0x7e17, 0x423d, { 0x9b, 0xc5, 0x12, 0x7d, 0x99, 0x65, 0xd9, 0xaf } };
class IBoundary : public IUnknown
{
public:

	ICALLTYPE CoverEdge(int nEdge, VARIANT_BOOL bCover)=0;
	ICALLTYPE GetCoverEdgeFlag(int nEdge, VARIANT_BOOL* pbCover)=0;
	ICALLTYPE GetEdgeCount(int* pnCount)=0;
	ICALLTYPE GetEdgePosition(int nEdge, double* pdCrossPosn)=0;
	ICALLTYPE GetEdge(int nEdge, IStrip **ppIStrip)=0;
	IUNKNOWN_
};

// {6B0C8DF3-2B3E-47bc-B0C3-A882A7C033E7}
static const GUID IID_IModel = { 0x6b0c8df3, 0x2b3e, 0x47bc, { 0xb0, 0xc3, 0xa8, 0x82, 0xa7, 0xc0, 0x33, 0xe7 } };
class IModel : public IUnknown
{
public:
	ICALLTYPE GetSectionCount(int* pnCount)=0;
	ICALLTYPE Save(char* szFilePath)=0;
	ICALLTYPE GetSection( int index, ISection **ppISection)=0;
	ICALLTYPE Update()=0;
	ICALLTYPE GetModelPath(BSTR *path)=0;
	ICALLTYPE GetItemCount(int* count)=0;
	ICALLTYPE GetItem(int index, IItem **iItem)=0;
	IUNKNOWN_
};

// {DF1A18A9-BCB4-4789-AD84-9D4BB415FA04}
static const GUID IID_IModel2 = { 0xdf1a18a9, 0xbcb4, 0x4789, { 0xad, 0x84, 0x9d, 0x4b, 0xb4, 0x15, 0xfa, 0x4 } };
class IModel2 : public IUnknown
{
public:
	ICALLTYPE GetCurrentObject(ISection ** iObject)=0;
	ICALLTYPE SetCurrentObject(ISection *iObject)=0;
	ICALLTYPE RemoveAll()=0;
	ICALLTYPE Add(ISection *object)=0;
	ICALLTYPE FindItem(KCSceneItemInfo* description, IItem **item)=0;
	ICALLTYPE AddNewItem(EObjectType eType, IItem **item)=0;
	ICALLTYPE Display(IUnknown* object, VARIANT_BOOL bPersistent, IItem **item)=0;
	ICALLTYPE Remove(IItem *item)=0;
	ICALLTYPE GetSceneItem(int index, IItem **item)=0;
	ICALLTYPE FindSection(ISection *iObject, VARIANT_BOOL searchChildren, int* indexFound)=0;
	ICALLTYPE Detach(IItem *item)=0;
	ICALLTYPE JoinCurrentSection(ISection *iObjectTo, VARIANT_BOOL changeCurrent)=0;
	ICALLTYPE JoinToCurrent(IItem *iItem, VARIANT_BOOL newItem)=0;
	ICALLTYPE AddCopy(ISection *obj, ISection** newCopy)=0;
	ICALLTYPE DeleteAll(EObjectType eType, VARIANT_BOOL includeChildren, int* countDeleted)=0;
	ICALLTYPE GetName(BSTR *pbstrName)=0;
	ICALLTYPE SetName(char* name)=0;
	ICALLTYPE AddItem(IItem *item)=0;
	IUNKNOWN_
};

// {EF1C88F4-A14B-41AE-B7B7-514052AF5FDF}
static const GUID IID_IModel_KC = { 0xef1c88f4, 0xa14b, 0x41ae, { 0xb7, 0xb7, 0x51, 0x40, 0x52, 0xaf, 0x5f, 0xdf } };
class IModel_KC : public IUnknown
{
public:
	ICALLTYPE GetItemCount(int* pnCount)=0;
	ICALLTYPE GetItem(int index, IItem **item)=0;
	ICALLTYPE Add(IItem *item)=0;
	ICALLTYPE Remove(IItem *item)=0;
	ICALLTYPE RemoveAll()=0;
	ICALLTYPE AddNewItem(EObjectType eType, IItem **item)=0;
	IUNKNOWN_
};

// {58BC3200-A439-4E5B-84FB-A9F6D5321A66}
static const GUID IID_IModelSearch_KC = { 0x58bc3200, 0xa439, 0x4e5b, { 0x84, 0xfb, 0xa9, 0xf6, 0xd5, 0x32, 0x1a, 0x66 } };
class IModelSearch_KC : public IUnknown
{
public:
	ICALLTYPE CreateDefaultDescription(IDictionary_KC** description)=0;
	ICALLTYPE FindItem(IDictionary_KC* description, IItem** iObject)=0;
	IUNKNOWN_
};

// {87A89B4E-86EE-421d-AC6E-727D8984AC82}
static const GUID IID_IArray = { 0x87a89b4e, 0x86ee, 0x421d, { 0xac, 0x6e, 0x72, 0x7d, 0x89, 0x84, 0xac, 0x82 } };
class IArray : public IUnknown
{
public:
	ICALLTYPE GetCount(int * pnCount)=0;
	ICALLTYPE GetAt(int index, IUnknown **ppIUnkn)=0;
	ICALLTYPE InsertAt(int indexAt, IUnknown * pIUnkn)=0;
	ICALLTYPE RemoveAt(int index)=0;
	ICALLTYPE Add(IUnknown * pIUnkn)=0;
	ICALLTYPE Find(IUnknown * pIUnkn, int* pnIndex)=0;
	ICALLTYPE RemoveAll()=0;
	IUNKNOWN_
};

// {183F9350-6DE4-4546-8117-0E0E81242D26}
static const GUID IID_IArray2 = { 0x183f9350, 0x6de4, 0x4546, { 0x81, 0x17, 0xe, 0xe, 0x81, 0x24, 0x2d, 0x26 } };
class IArray2 : public IUnknown
{
public:
	ICALLTYPE GetCount(int * pnCount)=0;
	ICALLTYPE GetAt(int index, VARIANT* val)=0;
	ICALLTYPE InsertAt(int indexAt, VARIANT* val)=0;
	ICALLTYPE RemoveAt(int index)=0;
	ICALLTYPE Add(VARIANT* val)=0;
	ICALLTYPE Find(VARIANT* val, int* pnIndex)=0;
	ICALLTYPE InsertNew(int type, int indexAt)=0;
	ICALLTYPE SetCount(int count)=0;
	ICALLTYPE SetAt(int index, VARIANT* val)=0;
	ICALLTYPE RemoveAll()=0;
	IUNKNOWN_
};

// {1D93969E-4DA3-4156-A48C-9A03F0FFDDA6}
static const GUID IID_IArrayNum = { 0x1d93969e, 0x4da3, 0x4156, { 0xa4, 0x8c, 0x9a, 0x3, 0xf0, 0xff, 0xdd, 0xa6 } };
class IArrayNum : public IUnknown
{
public:
	ICALLTYPE GetCount(int* count)=0;
	ICALLTYPE SetCount(int count)=0;
	ICALLTYPE Get(int index, double* val)=0;
	ICALLTYPE Set(int index, double val)=0;
	ICALLTYPE InsertAt(int index, double val)=0;
	ICALLTYPE Add(double val)=0;
	ICALLTYPE RemoveAt(int index)=0;
	ICALLTYPE RemoveAll()=0;
	IUNKNOWN_
};

// {8203C52D-EAD5-462d-BFDB-3D81A1E54B99}
static const GUID IID_IArray3D = { 0x8203c52d, 0xead5, 0x462d, { 0xbf, 0xdb, 0x3d, 0x81, 0xa1, 0xe5, 0x4b, 0x99 } };
class IArray3D : public IUnknown
{
public:
	ICALLTYPE GetCount(int* count)=0;
	ICALLTYPE SetCount(int count)=0;
	ICALLTYPE Get(int index, double* x, double* y, double* z)=0;
	ICALLTYPE Set(int index, double x, double y, double z)=0;
	ICALLTYPE InsertAt(int index, double x, double y, double z)=0;
	ICALLTYPE Add(double x, double y, double z)=0;
	ICALLTYPE RemoveAt(int index)=0;
	ICALLTYPE RemoveAll()=0;
	IUNKNOWN_
};

// {F934EEE0-899C-4e46-9324-A2835EA9E4A5}
static const GUID IID_IData = { 0xf934eee0, 0x899c, 0x4e46, { 0x93, 0x24, 0xa2, 0x83, 0x5e, 0xa9, 0xe4, 0xa5 } };
class IData : public IUnknown
{
public:
	ICALLTYPE GetRecordCount(int *count)=0;
	ICALLTYPE GetFieldCount(int *count)=0;
	ICALLTYPE GetFieldAt(int index, int field, VARIANT *data)=0;
	ICALLTYPE GetDoubleAt(int index, int field, double *data)=0;
	ICALLTYPE GetLongAt(int index, int field, long *data)=0;
	ICALLTYPE GetStringAt(int index, int field, BSTR *string)=0;
	ICALLTYPE GetFieldType(int field, EDataType* type)=0;
	ICALLTYPE GetFieldDescription(int field, BSTR *descript)=0;
	IUNKNOWN_
};

// {1CBE48BE-4BE1-4d74-B0BF-4B91358E3473}
static const GUID IID_IDataMods = { 0x1cbe48be, 0x4be1, 0x4d74, { 0xb0, 0xbf, 0x4b, 0x91, 0x35, 0x8e, 0x34, 0x73 } };
class IDataMods : public IUnknown
{
public:
	ICALLTYPE SetFieldAt(int index, int field, VARIANT *data)=0;
	ICALLTYPE SetDoubleAt(int index, int field, double data)=0;
	ICALLTYPE SetLongAt(int index, int field, long data)=0;
	ICALLTYPE SetStringAt(int index, int field, char *string)=0;
	IUNKNOWN_
};


// {D2C1E2C2-129F-42df-B0E3-3E61E54F3EAC}
static const GUID IID_IRecordSet = { 0xd2c1e2c2, 0x129f, 0x42df, { 0xb0, 0xe3, 0x3e, 0x61, 0xe5, 0x4f, 0x3e, 0xac } };
class IRecordSet : public IUnknown
{
public:
	ICALLTYPE InsertRecordAt(int index)=0;
	ICALLTYPE RemoveRecordAt(int index)=0;
	IUNKNOWN_
};

// {0AB681B0-A9C9-4be9-92DE-A6368781F0C9}
static const GUID IID_IDataTable = { 0xab681b0, 0xa9c9, 0x4be9, { 0x92, 0xde, 0xa6, 0x36, 0x87, 0x81, 0xf0, 0xc9 } };
class IDataTable : public IUnknown
{
public:
	ICALLTYPE GetFieldCount(int* count)=0;
	ICALLTYPE DeleteField(int field)=0;
	ICALLTYPE InsertField(int field, EDataType type)=0;
	ICALLTYPE SetFieldSize(int field, int size)=0;
	ICALLTYPE GetFieldSize(int field, int* size)=0;
	ICALLTYPE SetFieldDescription(int field, char* description)=0;
	IUNKNOWN_
};

// {E0D6C44D-78DB-4370-89DC-78EA05BF1F03}
static const GUID IID_IDataSource = { 0xe0d6c44d, 0x78db, 0x4370, { 0x89, 0xdc, 0x78, 0xea, 0x5, 0xbf, 0x1f, 0x3 } };
class IDataSource : public IUnknown
{
public:
	ICALLTYPE GetTable(char* name, IDataTable** table, VARIANT_BOOL* result )=0;
	ICALLTYPE CreateTable(char* name)=0;
	ICALLTYPE DropTable(char* name)=0;
	IUNKNOWN_
};

// {3CF8586E-5F87-4eaf-B221-3571E361556C}
static const GUID IID_IUserData = { 0x3cf8586e, 0x5f87, 0x4eaf, { 0xb2, 0x21, 0x35, 0x71, 0xe3, 0x61, 0x55, 0x6c } };
class IUserData : public IUnknown
{
public:
	ICALLTYPE GetData(int *data)=0;
	ICALLTYPE SetData(int data)=0;
	ICALLTYPE GetLayerName(BSTR *name)=0;
	ICALLTYPE SetLayerName(char* name)=0;
	IUNKNOWN_
};

// {2964D02D-E459-4b32-8B13-29891058D666}
static const GUID IID_ILayers = { 0x2964d02d, 0xe459, 0x4b32, { 0x8b, 0x13, 0x29, 0x89, 0x10, 0x58, 0xd6, 0x66 } };
class ILayers : public IUnknown
{
public:
	ICALLTYPE Set(char* name, int value)=0;
	ICALLTYPE Get(char* layer, VARIANT_BOOL * all, int* value)=0;
	ICALLTYPE GetCount(char* layer, int* count)=0;
	ICALLTYPE EnumLayers(IData** names)=0;
	ICALLTYPE GetLayerItems(char* layer, IArray** items)=0;
	IUNKNOWN_
};

// {38B04A5A-238E-4a85-B6CB-1AB0E2055227}
static const GUID IID_ISORSectionGenerator = { 0x38b04a5a, 0x238e, 0x4a85, { 0xb6, 0xcb, 0x1a, 0xb0, 0xe2, 0x5, 0x52, 0x27 } };
class ISORSectionGenerator : public IUnknown
{
public:
	ICALLTYPE Create(DISectInfo* pInfo, ISection **ppISection)=0;
	IUNKNOWN_
};

// {F96C40D5-069B-4b8b-A040-93B698E52A94}
static const GUID IID_I3DSectionGenerator = { 0xf96c40d5, 0x69b, 0x4b8b, { 0xa0, 0x40, 0x93, 0xb6, 0x98, 0xe5, 0x2a, 0x94 } };
class I3DSectionGenerator : public IUnknown
{
public:
	ICALLTYPE Create(DI3DSectInfo* pInfo,  ISection **ppISection)=0;
	IUNKNOWN_
};

// {CB56DAAC-0DDE-4d62-8367-9722582281F5}
static const GUID IID_IDIObjGenerator = { 0xcb56daac, 0xdde, 0x4d62, { 0x83, 0x67, 0x97, 0x22, 0x58, 0x22, 0x81, 0xf5 } };
class IDIObjGenerator : public IUnknown
{
public:
	ICALLTYPE Create(EObjectType eType, IUnknown **iResult) = 0;
	ICALLTYPE Create2(int eType, VARIANT* itemResult) = 0;
	ICALLTYPE Create3(char* type, IUnknown **iResult) = 0;
	ICALLTYPE Create4(char* type, IUnknown *iParam, IUnknown **iResult) = 0;
	IUNKNOWN_
};

// {E6440262-9C15-4df0-A0B5-62F8EEBBD831}
static const GUID IID_IDirectView = { 0xe6440262, 0x9c15, 0x4df0, { 0xa0, 0xb5, 0x62, 0xf8, 0xee, 0xbb, 0xd8, 0x31 } };
class IDirectView : public IUnknown
{
public:
	ICALLTYPE Attach(LONG hWnd)=0;
	ICALLTYPE Detach()=0;
	IUNKNOWN_
};

// {D07E372C-E0BD-4637-9231-98626E678B82}
static const GUID IID_IStripRender = { 0xd07e372c, 0xe0bd, 0x4637, { 0x92, 0x31, 0x98, 0x62, 0x6e, 0x67, 0x8b, 0x82 } };
class IStripRender : public IUnknown
{
public:
	ICALLTYPE GetJointRegularity( int nJoint, VARIANT_BOOL * bRegular )=0;
	ICALLTYPE SetRegularJoint( int nJoint, VARIANT_BOOL bRegular )=0;
	IUNKNOWN_
};

class IIteratorT : public IUnknown
{
public:
	ICALLTYPE GetCount(POSNT* count)=0;
	ICALLTYPE GetHeadPosition(POSNT* head)=0;
	ICALLTYPE GetNext(POSNT* position)=0;
	ICALLTYPE GetTailPosition(POSNT* tail)=0;
	ICALLTYPE GetPrev(POSNT* position)=0;
	IUNKNOWN_
};

class IListT : public IUnknown
{
public:
	ICALLTYPE GetIterator( IIteratorT** iterator )=0;
	ICALLTYPE GetAt(POSNT position, void **object )=0;
	IUNKNOWN_
};

class IListUnkn_KCT : public IUnknown
{
public:
	ICALLTYPE GetIterator( IIteratorT** iterator )=0;
	ICALLTYPE GetAt(POSNT position, IUnknown **item )=0;
	ICALLTYPE AddHead(IUnknown* item)=0;
	ICALLTYPE AddTail(IUnknown* item)=0;
	ICALLTYPE SetAt(POSNT position, IUnknown* item)=0;
	ICALLTYPE RemoveAt(POSNT position)=0;
	ICALLTYPE RemoveAll()=0;
	ICALLTYPE InsertBefore(POSNT position, IUnknown* item, POSNT* positionItem)=0;
	ICALLTYPE InsertAfter(POSNT position, IUnknown* item, POSNT* positionItem)=0;
	IUNKNOWN_
};

class IRelationT : public IUnknown
{
public:
	ICALLTYPE GetRelation( IListT** pairs )=0;
	ICALLTYPE GetItemList( VARIANT_BOOL bFirstSet, IListT** items )=0;
	IUNKNOWN_
};

// {27AECDF5-2BCD-4e5a-B732-DA832B6F1209}
static const GUID IID_IObject_KC = { 0x27aecdf5, 0x2bcd, 0x4e5a, { 0xb7, 0x32, 0xda, 0x83, 0x2b, 0x6f, 0x12, 0x9 } };
class IObject_KC : public IUnknown
{
public:
	ICALLTYPE IsEqual(IObject_KC* object, VARIANT_BOOL* equal) = 0;
	ICALLTYPE GetHash(int* hash) = 0;
	ICALLTYPE Copy(IObject_KC* object) = 0;
	ICALLTYPE Replicate(IObject_KC** object) = 0;
	ICALLTYPE Convert(int typeID, IObject_KC** object) = 0;
	ICALLTYPE Execute(char* operation, IDictionary_KC* context, VARIANT_BOOL* success)=0;
	IUNKNOWN_
};

// {3D0EF9EA-273B-4129-AA43-58FBFBC3BE75}
static const GUID IID_IBox = { 0x3d0ef9ea, 0x273b, 0x4129, { 0xaa, 0x43, 0x58, 0xfb, 0xfb, 0xc3, 0xbe, 0x75 } };
class IBox : public IUnknown
{
public:
	ICALLTYPE GetRange(int axis, double *min, double *max)=0;
	ICALLTYPE SetRange(int axis, double min, double max)=0;
	IUNKNOWN_
};

class IMetricsT : public IUnknown
{
public:
	ICALLTYPE Dist( IMetricsT * iIMetricObj, MetrDistContext* context, IRelationT** nearest, double* dist )=0;
	ICALLTYPE GetDimensions( DIPoint* min, DIPoint* max )=0;
	ICALLTYPE GetPointLocation( DIPoint* pt, int* location )=0;
	ICALLTYPE IsCloserThan( IMetricsT * iIMetricObj, double dist, MetrIsCloserThanContext* context, VARIANT_BOOL* closer )=0;
	IUNKNOWN_
};

// {1FAEFCB1-F3C8-4d37-A37D-04E76CDB80E3}
static const GUID IID_IMetrics2 = { 0x1faefcb1, 0xf3c8, 0x4d37, { 0xa3, 0x7d, 0x4, 0xe7, 0x6c, 0xdb, 0x80, 0xe3 } };
class IMetrics2 : public IUnknown
{
public:
	ICALLTYPE GetLocalDimensions(KCBox* localBox, KCBox* dims)=0;
	ICALLTYPE Dist( IMetrics2 * iIMetricObj, DIPoint* ptThis, DIPoint* ptOther, double* dist )=0;
	ICALLTYPE DistPoint( DIPoint* pt, DIPoint* ptNearest, double* dist )=0;
	IUNKNOWN_
};

// {B25EDA37-A9CB-493B-8F86-032BB11586E3}
static const GUID IID_IPointCloud_KC = { 0xb25eda37, 0xa9cb, 0x493b, { 0x8f, 0x86, 0x3, 0x2b, 0xb1, 0x15, 0x86, 0xe3 } };
class IPointCloud_KC : public IUnknown
{
public:
	ICALLTYPE GetAxisAlignedBoundingBox(VARIANT_BOOL localAxes, IBox** box)=0;
	ICALLTYPE GetOrientedBoundingBox(IBox* box)=0;
	ICALLTYPE FitOrientedBoundingBoxAveragesMethod( int* result, IBox** box)=0;
	ICALLTYPE GetMinimumOrientedBoundingBox( float tolerance, int performanceVsPrecision, int* result, IBox** box)=0;
	ICALLTYPE GetMeanPoint(DIPoint* mean)=0;
	ICALLTYPE ReduceData(VARIANT_BOOL leave, int start, int step, int end)=0;
	IUNKNOWN_
};


#ifndef _NET
// {88AE6AEE-6280-4a9f-9FE3-FF2331857A1A}
static const GUID IID_ISymmetry = { 0x88ae6aee, 0x6280, 0x4a9f, { 0x9f, 0xe3, 0xff, 0x23, 0x31, 0x85, 0x7a, 0x1a } };
class ISymmetry : public IUnknown
{
public:
	ICALLTYPE GetType(char* szType) = 0;
	ICALLTYPE IsEnabled(VARIANT_BOOL * pbEnabled) = 0;
	ICALLTYPE Enable(VARIANT_BOOL bEnable) = 0;
	ICALLTYPE GetRepeatCount(int * pnRepeat) = 0;
	ICALLTYPE SetRepeatCount(int nRepeat) = 0;
	ICALLTYPE IsVisible(VARIANT_BOOL * pbVisible) = 0;
	ICALLTYPE SetVisible(VARIANT_BOOL bVisible) = 0;
	ICALLTYPE SetIncludeChildren(VARIANT_BOOL bInclude) = 0;
	ICALLTYPE GetIncludeChildren(VARIANT_BOOL * pbInclude) = 0;
	IUNKNOWN_
};
#endif

// {A73C2F06-7251-4dde-9C5D-7B5F16CC61B5}
static const GUID IID_ISymmetryStd = { 0xa73c2f06, 0x7251, 0x4dde, { 0x9c, 0x5d, 0x7b, 0x5f, 0x16, 0xcc, 0x61, 0xb5 } };
class ISymmetryStd : public IUnknown
{
public:
	ICALLTYPE GetType(char* szType) = 0;
	ICALLTYPE IsEnabled(VARIANT_BOOL * pbEnabled) = 0;
	ICALLTYPE Enable(VARIANT_BOOL bEnable) = 0;
	ICALLTYPE GetRepeatCount(int * pnRepeat) = 0;
	ICALLTYPE SetRepeatCount(int nRepeat) = 0;
	ICALLTYPE IsVisible(VARIANT_BOOL * pbVisible) = 0;
	ICALLTYPE SetVisible(VARIANT_BOOL bVisible) = 0;
	ICALLTYPE SetIncludeChildren(VARIANT_BOOL bInclude) = 0;
	ICALLTYPE GetIncludeChildren(VARIANT_BOOL * pbInclude) = 0;
	IUNKNOWN_
};

// {70CD5E71-8E0A-4cb6-A7F2-77DF0A124D09}
static const GUID IID_IAxialSymmetry = { 0x70cd5e71, 0x8e0a, 0x4cb6, { 0xa7, 0xf2, 0x77, 0xdf, 0xa, 0x12, 0x4d, 0x9 } };
class IAxialSymmetry : public ISymmetry
{
public:
	ICALLTYPE GetAngle(double* pdAngle) = 0;
	ICALLTYPE SetAngle(double dAngle) = 0;
	IUNKNOWN_
};

// {8C34EA28-E95B-49ec-9763-3AC8CB5F092D}
static const GUID IID_ISymmetries = { 0x8c34ea28, 0xe95b, 0x49ec, { 0x97, 0x63, 0x3a, 0xc8, 0xcb, 0x5f, 0x9, 0x2d } };
class ISymmetries : public IUnknown
{
public:
	ICALLTYPE GetCount(int * pnCount) = 0;
	ICALLTYPE GetSymmetry(int index, ISymmetry **ppISymmetry) = 0;
	IUNKNOWN_
};

// {4D37E0D8-CBA3-46ee-9EC2-B60AF01B78E5}
static const GUID IID_ILocation = { 0x4d37e0d8, 0xcba3, 0x46ee, { 0x9e, 0xc2, 0xb6, 0xa, 0xf0, 0x1b, 0x78, 0xe5 } };
class ILocation : public IUnknown
{
public:
	ICALLTYPE GetLocation(double * pdX, double * pdY, double * pdZ) = 0;
	ICALLTYPE SetLocation(double dX, double dY, double dZ) = 0;
	IUNKNOWN_
};

// {2D434EF2-D518-4d3c-AA7B-67E1117D06BD}
static const GUID IID_ILocationEx = { 0x2d434ef2, 0xd518, 0x4d3c, { 0xaa, 0x7b, 0x67, 0xe1, 0x11, 0x7d, 0x6, 0xbd } };
class ILocationEx : public IUnknown
{
public:
	ICALLTYPE GetFrameType(int* frame) = 0;
	ICALLTYPE SetFrameType(int frame) = 0;
	ICALLTYPE GetDepthPositionType(int* type) = 0;
	ICALLTYPE SetDepthPositionType(int type) = 0;
	IUNKNOWN_
};

// {11AC41C1-C326-4e40-81F6-A76C42CA4D25}
static const GUID IID_IAxis = { 0x11ac41c1, 0xc326, 0x4e40, { 0x81, 0xf6, 0xa7, 0x6c, 0x42, 0xca, 0x4d, 0x25 } };
class IAxis : public ILocation
{
public:
	ICALLTYPE GetDirection(double * pdVX, double * pdVY, double * pdVZ) = 0;
	ICALLTYPE SetDirection(double dVX, double dVY, double dVZ) = 0;
	IUNKNOWN_
};

// {77FBA452-3D2B-4361-99BC-6381A57E4885}
static const GUID IID_I3Dpt = { 0x77fba452, 0x3d2b, 0x4361, { 0x99, 0xbc, 0x63, 0x81, 0xa5, 0x7e, 0x48, 0x85 } };
class I3Dpt : public IUnknown
{
public:
	ICALLTYPE Get(double* x, double* y, double* z) = 0;
	ICALLTYPE Set(double x, double y, double z) = 0;
	ICALLTYPE GetCoord(short coord, double * val) = 0;
	ICALLTYPE SetCoord(short coord, double val) = 0;
	ICALLTYPE Add(I3Dpt* v) = 0;
	ICALLTYPE Subtr(I3Dpt* v) = 0;
	ICALLTYPE Mult(double m) = 0;
	ICALLTYPE Copy(I3Dpt* p) = 0;
	ICALLTYPE Norm(double* norm) = 0;
	ICALLTYPE Normalize(VARIANT_BOOL* result) = 0;
	ICALLTYPE Dist(I3Dpt* p, double* dist) = 0;
	ICALLTYPE Product(I3Dpt* v, double* product) = 0;
	ICALLTYPE CrossProduct(I3Dpt* v, I3Dpt* u) = 0;
	ICALLTYPE Angle(I3Dpt* v, double* angle) = 0;
	IUNKNOWN_
};

// {2B618C81-3F5D-48ff-810A-CAA769E72133}
static const GUID IID_IMaterial = { 0x2b618c81, 0x3f5d, 0x48ff, { 0x81, 0xa, 0xca, 0xa7, 0x69, 0xe7, 0x21, 0x33 } };
class IMaterial : public IUnknown
{
public:
	ICALLTYPE GetColor(int nComponent, float * pfRed, float * pfGreen, float * pfBlue) = 0;
	ICALLTYPE SetColor(int nComponent, float fRed, float fGreen, float fBlue) = 0;
	ICALLTYPE GetAlpha(int nComponent, float * pfAlpha) = 0;
	ICALLTYPE SetAlpha(int nComponent, float fAlpha) = 0;
	ICALLTYPE GetShininess(float * pfShininess) = 0;
	ICALLTYPE SetShininess(float fShininess) = 0;
	ICALLTYPE IsLightingEnabled(VARIANT_BOOL* enabled) = 0;
	ICALLTYPE SetLightingEnabled(VARIANT_BOOL enabled) = 0;
	ICALLTYPE GetUseForWire(VARIANT_BOOL* enabled) = 0;
	ICALLTYPE SetUseForWire(VARIANT_BOOL enabled) = 0;
	ICALLTYPE GetWireFrameColor(float* red, float* green, float* blue) = 0;
	ICALLTYPE SetWireFrameColor(float red, float green, float blue) = 0;
	ICALLTYPE GetUseGlobalWireColor(VARIANT_BOOL* bUse) = 0;
	ICALLTYPE SetUseGlobalWireColor(VARIANT_BOOL bUse) = 0;
	IUNKNOWN_
};

class IClipPlaneT : public IUnknown
{
public:
	ICALLTYPE IsEnabled(VARIANT_BOOL * pbEnabled) = 0;
	ICALLTYPE Enable(VARIANT_BOOL bEnable) = 0;
	ICALLTYPE GetNormal(IAxis **ppIAxis) = 0;
	ICALLTYPE Share(VARIANT_BOOL bShare) = 0;
	ICALLTYPE GetID(POSNT *plID) = 0;
	ICALLTYPE UseShared(POSNT lID) = 0;
	ICALLTYPE SelectSide(VARIANT_BOOL bPositive) = 0;
	ICALLTYPE SetClipChildren(VARIANT_BOOL bClip) = 0;
	ICALLTYPE GetClipChildren(VARIANT_BOOL * pbClip) = 0;
	ICALLTYPE Show(VARIANT_BOOL show) = 0;
	ICALLTYPE GetShow(VARIANT_BOOL *show) = 0;
	IUNKNOWN_
};


// {D7F3B37E-CE19-4205-BE11-EE5BCF60BB2A}
static const GUID IID_IClipArray = { 0xd7f3b37e, 0xce19, 0x4205, { 0xbe, 0x11, 0xee, 0x5b, 0xcf, 0x60, 0xbb, 0x2a } };
class IClipArray : public IUnknown
{
public:
	ICALLTYPE IsEnabled(int nPlane, VARIANT_BOOL * pbEnabled) = 0;
	ICALLTYPE Enable(int nPlane, VARIANT_BOOL bEnable) = 0;
	ICALLTYPE GetClipPlane(int nPlane, IClipPlaneT **ppIPlane) = 0;
	IUNKNOWN_
};

class IGraphicItem;

// {46F86BAC-A9BD-467b-A3B0-C5B94D106C84}
static const GUID IID_IView = { 0x46f86bac, 0xa9bd, 0x467b, { 0xa3, 0xb0, 0xc5, 0xb9, 0x4d, 0x10, 0x6c, 0x84 } };
class	IView : public IUnknown
{
public:
	ICALLTYPE GetModel(IModel **ppIModel) = 0;
	ICALLTYPE GetGlobalAxesFlag(VARIANT_BOOL *pbShow) = 0;
	ICALLTYPE ShowGlobalAxes(VARIANT_BOOL bShow) = 0;
	ICALLTYPE GetBackColor(long *plCOLORREF) = 0;
	ICALLTYPE SetBackColor(long lCOLORREF) = 0;
	ICALLTYPE GetRenderType(int *pnType) = 0;
	ICALLTYPE SetRenderType(int nType) = 0;
	ICALLTYPE Update() = 0;
	ICALLTYPE Enable(int nFlags, VARIANT_BOOL bEnable) = 0;
	ICALLTYPE GetEnableFlag(int nFlag, VARIANT_BOOL *pbEnable) = 0;
	ICALLTYPE Reset(VARIANT_BOOL redraw) = 0;
	ICALLTYPE ResetEx(EViewReset action) = 0;
	ICALLTYPE GetViewType(int *type) = 0;
	ICALLTYPE SetViewType(int type) = 0;
	ICALLTYPE AddExternalModel(IModel *model) = 0;
	ICALLTYPE ShowHideAll(VARIANT_BOOL show) = 0;
	ICALLTYPE GoTo(IItem* iObject) = 0;
	ICALLTYPE GetGraphicItem(IObject_KC* object, IGraphicItem** item) = 0;
	ICALLTYPE GetProjectionType(int* type) = 0;
	ICALLTYPE SetProjectionType(int type) = 0;
	IUNKNOWN_
};

// {C9F06650-B713-49dc-9FFF-354D04E2ACBC}
static const GUID IID_IView2 = { 0xc9f06650, 0xb713, 0x49dc, { 0x9f, 0xff, 0x35, 0x4d, 0x4, 0xe2, 0xac, 0xbc } };
class IView2 : public IUnknown
{
public:
	ICALLTYPE Enable(int nFlags, VARIANT_BOOL bEnable) = 0;
	ICALLTYPE GetEnableFlag(int nFlag, VARIANT_BOOL *pbEnable) = 0;
	ICALLTYPE Reset(VARIANT_BOOL redraw) = 0;
	ICALLTYPE ResetEx(EViewReset action) = 0;
	ICALLTYPE GetViewType(int *type) = 0;
	ICALLTYPE SetViewType(int type) = 0;
	ICALLTYPE AddExternalModel(IModel *model) = 0;
	ICALLTYPE ShowHideAll(VARIANT_BOOL show) = 0;
	ICALLTYPE GoTo(IItem* iObject) = 0;
	ICALLTYPE GetGraphicItem(IObject_KC* object, IGraphicItem** item) = 0;
	IUNKNOWN_
};

// {84E6906D-DB03-426e-8D47-E722DF306D68}
static const GUID IID_ILook = { 0x84e6906d, 0xdb03, 0x426e, { 0x8d, 0x47, 0xe7, 0x22, 0xdf, 0x30, 0x6d, 0x68 } };
class ILook : public IUnknown
{
public:
	ICALLTYPE Translate(double vx, double vy, double vz) = 0;
	ICALLTYPE Rotate(double angle, double vx, double vy, double vz) = 0;
	ICALLTYPE Zoom(double scale) = 0;
	ICALLTYPE ZoomEx(double scale, double centerx, double centery, double centerz) = 0;
	ICALLTYPE GetViewingVolume(int nAxis, double *pdMin, double *pdMax) = 0;
	ICALLTYPE SetViewingVolume(int nAxis, double dMin, double dMax) = 0;
	IUNKNOWN_
};

// {FA65F017-CEB7-4b13-ABAC-3535B338928D}
static const GUID IID_ILookEx = { 0xfa65f017, 0xceb7, 0x4b13, { 0xab, 0xac, 0x35, 0x35, 0xb3, 0x38, 0x92, 0x8d } };
class ILookEx : public IUnknown
{
public:
	ICALLTYPE ScreenTo3D(int xWnd, int yWnd, DIPoint *pPt) = 0;
	ICALLTYPE ProjectToScreen(DIPoint pt, int *pxWnd, int *pyWnd) = 0;
	ICALLTYPE ScreenTo3DEx(int xWnd, int yWnd, double dist, DIPoint *pPt) = 0;
	ICALLTYPE ScreenToEyeFrame(int xWnd, int yWnd, double* x, double* y) = 0;
	IUNKNOWN_
};

class IMeshPointKCT : public IUnknown
{
public:
	ICALLTYPE GetPoint(DIPoint* point) = 0;
	ICALLTYPE GetLocation(EPointInSimplexLocation* location) = 0;
	ICALLTYPE GetEdge(int* edge) = 0;
	ICALLTYPE GetSimplexPosition(POSNT* position) = 0;
	IUNKNOWN_
};

// {27FB24BD-383D-41B9-8871-29684914624D}
static const GUID IID_ISurfacePointKC = { 0x27fb24bd, 0x383d, 0x41b9, { 0x88, 0x71, 0x29, 0x68, 0x49, 0x14, 0x62, 0x4d } };
class ISurfacePointKC : public IUnknown
{
public:
	ICALLTYPE GetPoint(DIPoint* point) = 0;
	ICALLTYPE GetItem(IItem** item) = 0;
	ICALLTYPE GetNormalBasic(DIVect* normal) = 0;
	ICALLTYPE GetOffsetPoint(double offset, DIPoint* point) = 0;
	IUNKNOWN_
};

// {EF04A127-0774-4BD8-B191-23BFBF831D6A}
static const GUID IID_ISurfacePointArrayKC = { 0xef04a127, 0x774, 0x4bd8, { 0xb1, 0x91, 0x23, 0xbf, 0xbf, 0x83, 0x1d, 0x6a } };
class ISurfacePointArrayKC : public IUnknown
{
public:
	ICALLTYPE GetCount(int* count) = 0;
	ICALLTYPE GetPoint(int i, ISurfacePointKC** point) = 0;
	ICALLTYPE Extend(ISurfacePointArrayKC* extension) = 0;
	ICALLTYPE Offset(double offset) = 0;
	IUNKNOWN_
};

// {EE7DD1D9-38FF-4ac8-AD4F-899236993F56}
static const GUID IID_IPick = { 0xee7dd1d9, 0x38ff, 0x4ac8, { 0xad, 0x4f, 0x89, 0x92, 0x36, 0x99, 0x3f, 0x56 } };
class IPick : public IUnknown
{
public:
	ICALLTYPE GetItemAt(int xWnd, int yWnd, IItem** item) = 0;
	ICALLTYPE GetItemAt3D(int xWnd, int yWnd, DIPoint* point, IItem** item) = 0;
	ICALLTYPE GetItemAtDepth(int xWnd, int yWnd, float* minZ, float* maxZ, IItem** item) = 0;
	ICALLTYPE GetSurfacePointAt(int xWnd, int yWnd, ISurfacePointKC** surfPoint) = 0;
	IUNKNOWN_
};

// {1506004C-5A36-48fb-AC60-DB69D16AEF1E}
static const GUID IID_ISphere_KC = { 0x1506004c, 0x5a36, 0x48fb, { 0xac, 0x60, 0xdb, 0x69, 0xd1, 0x6a, 0xef, 0x1e } };
class ISphere_KC : public IUnknown
{
public:
	ICALLTYPE GetRadius(double* R) = 0;
	ICALLTYPE SetRadius(double R) = 0;
	ICALLTYPE GetLocation(double *x, double *y, double *z) = 0;
	ICALLTYPE SetLocation(double x, double y, double z) = 0;
	IUNKNOWN_
};

// {3AB63D05-C733-483d-9B6C-B25CFA36C080}
static const GUID IID_IFrameArray = { 0x3ab63d05, 0xc733, 0x483d, { 0x9b, 0x6c, 0xb2, 0x5c, 0xfa, 0x36, 0xc0, 0x80 } };
class IFrameArray : public IUnknown
{
public:
	ICALLTYPE GetCount(int* count) = 0;
	ICALLTYPE SetCount(int count) = 0;
	ICALLTYPE GetFrame(int index, IFrame** frame) = 0;
	ICALLTYPE SetFrame(int index, IFrame* frame) = 0;
	ICALLTYPE InsertAt(int index, IFrame* frame) = 0;
	ICALLTYPE Add(IFrame* frame) = 0;
	ICALLTYPE RemoveAt(int index) = 0;
	ICALLTYPE RemoveAll() = 0;
	IUNKNOWN_
};

// {7D46792B-087D-48bb-AC19-A69370DE79B4}
static const GUID IID_IViewModal = { 0x7d46792b, 0x87d, 0x48bb, { 0xac, 0x19, 0xa6, 0x93, 0x70, 0xde, 0x79, 0xb4 } };
class IViewModal : public IUnknown
{
public:
	ICALLTYPE SetViewMode(int nMode) = 0;
	ICALLTYPE GetViewMode(int* pnMode) = 0;
	ICALLTYPE SetCurrentFrame(IFrame *pIFrame) = 0;
	ICALLTYPE GetCurrentFrame(IFrame **ppIFrame) = 0;
	ICALLTYPE SetAutoMode(int nMode) = 0;
	ICALLTYPE GetAutoMode(int* pnMode) = 0;
	ICALLTYPE ResetAutoMode() = 0;
	IUNKNOWN_
};

// {1BC40D5D-791E-4e5a-814D-5F8C3B1E7B21}
static const GUID IID_IConstraint = { 0x1bc40d5d, 0x791e, 0x4e5a, { 0x81, 0x4d, 0x5f, 0x8c, 0x3b, 0x1e, 0x7b, 0x21 } };
class IConstraint : public IUnknown
{
public:
	ICALLTYPE SetFreedom(int nMovementType, int nDims) = 0;
	ICALLTYPE GetFreedom(int nMovementType, int *pnDims) = 0;
	ICALLTYPE GetConstraint(int nMovementType, IAxis **ppIAxis) = 0;
	ICALLTYPE Enable(bool bEnable) = 0;
	ICALLTYPE IsEnabled(bool *pbEnabled) = 0;
	IUNKNOWN_
};

// {8081A3B2-EDAE-433f-9FC8-0B438C0E6D2B}
static const GUID IID_ITexture = { 0x8081a3b2, 0xedae, 0x433f, { 0x9f, 0xc8, 0xb, 0x43, 0x8c, 0xe, 0x6d, 0x2b } };
class ITexture : public IUnknown
{
public:
	ICALLTYPE Load(char *szFilePath) = 0;
	ICALLTYPE IsLoaded(VARIANT_BOOL *pbLoaded) = 0;
	ICALLTYPE Unload() = 0;
	ICALLTYPE Enable(VARIANT_BOOL bEnable) = 0;
	ICALLTYPE IsEnabled(VARIANT_BOOL *pbEnabled) = 0;
	ICALLTYPE Blend(VARIANT_BOOL bBlend) = 0;
	ICALLTYPE GetBlendFlag(VARIANT_BOOL *pbBlend) = 0;
	ICALLTYPE UseTexture(ITexture* pITexture) = 0;
	ICALLTYPE LoadFromHandle(int handle) = 0;
	ICALLTYPE GetFileName(BSTR *name) = 0;
	IUNKNOWN_
};

// {3092747C-E7E6-40ab-B6CD-6D16E90C2E86}
static const GUID IID_IStripTopol = { 0x3092747c, 0xe7e6, 0x40ab, { 0xb6, 0xcd, 0x6d, 0x16, 0xe9, 0xc, 0x2e, 0x86 } };
class IStripTopol : public IUnknown
{
public:
	ICALLTYPE Break(int index) = 0;
	ICALLTYPE Delete(int index, VARIANT_BOOL segment) = 0;
	ICALLTYPE SetType(int index, ESegmentType eType) = 0;
	ICALLTYPE CloseLoop() = 0;
	ICALLTYPE Add(double x, double y, VARIANT_BOOL tail) = 0;
	ICALLTYPE DeleteAll() = 0;
	IUNKNOWN_
};

#ifndef _NET
// {CB17987A-02DA-4da5-A4DF-181EC765CBD4}
static const GUID IID_IPropertyArray = { 0xcb17987a, 0x2da, 0x4da5, { 0xa4, 0xdf, 0x18, 0x1e, 0xc7, 0x65, 0xcb, 0xd4 } };
class IPropertyArray : public IUnknown
{
public:
	ICALLTYPE GetCount(int * pnCount) = 0;
	ICALLTYPE GetProperty(char* szPropertyName, IUnknown **ppIUnkn) = 0;
	ICALLTYPE GetName(int index, char*  szName) = 0;
	ICALLTYPE SetProperty(char* szPropertyName, IUnknown *iUnknown) = 0;
	ICALLTYPE GetPropertyEx(char* szPropertyName, int par0, int par1, IUnknown **ppIUnkn) = 0;
	IUNKNOWN_
};
#endif

// {8A47DFEA-CE95-417b-BC06-B7A5ED83B9B5}
static const GUID IID_IPropertyArrayStd = { 0x8a47dfea, 0xce95, 0x417b, { 0xbc, 0x6, 0xb7, 0xa5, 0xed, 0x83, 0xb9, 0xb5 } };
class IPropertyArrayStd : public IUnknown
{
public:
	ICALLTYPE GetCount(int * pnCount) = 0;
	ICALLTYPE GetProperty(char* szPropertyName, IUnknown **ppIUnkn) = 0;
	ICALLTYPE GetName(int index, char*  szName) = 0;
	ICALLTYPE SetProperty(char* szPropertyName, IUnknown *iUnknown) = 0;
	IUNKNOWN_
};

// {3C208520-4152-4edf-8795-24EBBE1E883D}
static const GUID IID_IPropertyArray2 = { 0x3c208520, 0x4152, 0x4edf, { 0x87, 0x95, 0x24, 0xeb, 0xbe, 0x1e, 0x88, 0x3d } };
class IPropertyArray2 : public IUnknown
{
public:
	ICALLTYPE GetCount(int * pnCount) = 0;
	ICALLTYPE GetProperty(char* szPropertyName, VARIANT* prop) = 0;
	ICALLTYPE GetName(int index, BSTR *pbstrName) = 0;
	IUNKNOWN_
};

// {DE2363EB-71A4-440c-84D8-FA4095BE6A38}
static const GUID IID_ILightSource = { 0xde2363eb, 0x71a4, 0x440c, { 0x84, 0xd8, 0xfa, 0x40, 0x95, 0xbe, 0x6a, 0x38 } };
class ILightSource : public IUnknown
{
public:
	ICALLTYPE GetLocation(float * pdX, float * pdY, float * pdZ, VARIANT_BOOL *pbDirection) = 0;
	ICALLTYPE SetLocation(float dX, float dY, float dZ, VARIANT_BOOL bDirection) = 0;
	ICALLTYPE GetColor(int nComponent, float * pfRed, float * pfGreen, float * pfBlue) = 0;
	ICALLTYPE SetColor(int nComponent, float fRed, float fGreen, float fBlue) = 0;
	IUNKNOWN_
};

// {C3B852CF-0E85-476a-A4DF-3E3141965AF8}
static const GUID IID_ILine2D = { 0xc3b852cf, 0xe85, 0x476a, { 0xa4, 0xdf, 0x3e, 0x31, 0x41, 0x96, 0x5a, 0xf8 } };
class ILine2D : public IUnknown
{
public:
	ICALLTYPE GetEnd(int end, double* x, double* y) = 0;
	ICALLTYPE SetEnd(int end, double x, double y) = 0;
	IUNKNOWN_
};

// {26281740-2DCA-4850-9B1C-256202B4DB43}
static const GUID IID_IArc = { 0x26281740, 0x2dca, 0x4850, { 0x9b, 0x1c, 0x25, 0x62, 0x2, 0xb4, 0xdb, 0x43 } };
class IArc : public IUnknown
{
public:
	ICALLTYPE GetCurvature(double* curv) = 0;
	ICALLTYPE SetCurvature(double curv) = 0;
	ICALLTYPE Init(VARIANT_BOOL *success) = 0;
	ICALLTYPE IsValid(VARIANT_BOOL *valid) = 0;
	ICALLTYPE MakeValid(VARIANT_BOOL *valid) = 0;
	ICALLTYPE SetEndPoint(int end, double x, double y) = 0;
	IUNKNOWN_
};

// {1E974073-1140-4f74-8A5E-63D9DC052A94}
static const GUID IID_IArcEx = { 0x1e974073, 0x1140, 0x4f74, { 0x8a, 0x5e, 0x63, 0xd9, 0xdc, 0x5, 0x2a, 0x94 } };
class IArcEx : public IUnknown
{
public:
	ICALLTYPE GetPoint(double t, double* x, double* y) = 0;
	ICALLTYPE GetRadius(double* r) = 0;
	ICALLTYPE SetRadius(double r) = 0;
	ICALLTYPE GetCenter(double* x, double* y) = 0;
	ICALLTYPE GetAngleRange(double* a, double* b) = 0;
	ICALLTYPE PassThroughPoint(double x, double y, int* result) = 0;
	IUNKNOWN_
};

// {0844DBEB-C5EC-4287-B01F-B68F365943EA}
static const GUID IID_IStripArcLine_KC = { 0x844dbeb, 0xc5ec, 0x4287, { 0xb0, 0x1f, 0xb6, 0x8f, 0x36, 0x59, 0x43, 0xea } };
class IStripArcLine_KC : public IUnknown
{
public:
	ICALLTYPE AddPoint(double x, double y, VARIANT_BOOL arc, VARIANT_BOOL head, IElement** point) = 0;
	ICALLTYPE CloseStrip(VARIANT_BOOL arc, ILine2D** link) = 0;
	ICALLTYPE Clear() = 0;
	ICALLTYPE DeletePoint(IElement* point) = 0;
	ICALLTYPE DeleteLine(ILine2D* line) = 0;
	ICALLTYPE GetLength(double* length) = 0;
	ICALLTYPE GetDistPoint(double x, double y, KC2D* pointThis, double* dist) = 0;
	ICALLTYPE GetOrientation(KC3State* ccw) = 0;
	ICALLTYPE GetPointLocation(double x, double y, EPointLocation2D* location) = 0;
	ICALLTYPE Reverse() = 0;
	ICALLTYPE Sample(double deviation, IArrayNum** points) = 0;
	IUNKNOWN_
};

// {2A2C1450-296B-447d-9061-097BA0087E3B}
static const GUID IID_ILineMetr = { 0x2a2c1450, 0x296b, 0x447d, { 0x90, 0x61, 0x9, 0x7b, 0xa0, 0x8, 0x7e, 0x3b } };
class ILineMetr : public IUnknown
{
public:
	ICALLTYPE IntersectObject(ISection* iObject, IData** iPoints) = 0;
	ICALLTYPE IntersectModel(IModel* iModel, IData** iPoints) = 0;
	IUNKNOWN_
};

// {897B077A-06EC-482a-88F8-704DCCB2A010}
static const GUID IID_ILineMetr2 = { 0x897b077a, 0x6ec, 0x482a, { 0x88, 0xf8, 0x70, 0x4d, 0xcc, 0xb2, 0xa0, 0x10 } };
class ILineMetr2 : public IUnknown
{
public:
	ICALLTYPE IntersectObject(ISection* iObject, IData** iPoints) = 0;
	ICALLTYPE IntersectModel(IModel* iModel, IData** iPoints) = 0;
	IUNKNOWN_
};

// {A638DD14-D290-4e18-BFD5-9D041D287ADC}
static const GUID IID_ICurve = { 0xa638dd14, 0xd290, 0x4e18, { 0xbf, 0xd5, 0x9d, 0x4, 0x1d, 0x28, 0x7a, 0xdc } };
class ICurve : public IUnknown
{
public:
	ICALLTYPE  GetPoint(double t, double* x, double* y, double* z) = 0;
	ICALLTYPE  IsClosed(VARIANT_BOOL *closed) = 0;
	ICALLTYPE  SetClosed(VARIANT_BOOL closed) = 0;
	ICALLTYPE  GetLength(double* length) = 0;
	IUNKNOWN_
};

// {4BA32275-28F5-4c89-8AC4-71A6D9B407D9}
static const GUID IID_ICurveFreeForm = { 0x4ba32275, 0x28f5, 0x4c89, { 0x8a, 0xc4, 0x71, 0xa6, 0xd9, 0xb4, 0x7, 0xd9 } };
class ICurveFreeForm : public IUnknown
{
public:
	ICALLTYPE  GetData(IArray3D** points) = 0;
	ICALLTYPE  GetInterpolationDegree(int* degree) = 0;
	ICALLTYPE  SetInterpolationDegree(int degree) = 0;
	ICALLTYPE  Extend(ICurveFreeForm* curve) = 0;
	IUNKNOWN_
};

// {6990399B-8D3C-433e-8C69-215DFEC1C882}
static const GUID IID_IArcSpline3DCurve = { 0x6990399b, 0x8d3c, 0x433e, { 0x8c, 0x69, 0x21, 0x5d, 0xfe, 0xc1, 0xc8, 0x82 } };
class IArcSpline3DCurve : public IUnknown
{
public:
	ICALLTYPE  GetSplineNormals(IArray3D** normals) = 0;
	ICALLTYPE  GetCurvatureArray(IArrayNum** curvatures) = 0;
	ICALLTYPE  GetSpline(int index, IArc** iArc) = 0;
	ICALLTYPE  MakeSmoothModifyCurvatures() = 0;
	ICALLTYPE  MakeSmoothAlternating(double curvature) = 0;
	IUNKNOWN_
};

// {AF628DE3-5F3D-4841-9A1E-3C4369362B54}
static const GUID IID_ICurveFreeFormEx = { 0xaf628de3, 0x5f3d, 0x4841, { 0x9a, 0x1e, 0x3c, 0x43, 0x69, 0x36, 0x2b, 0x54 } };
class ICurveFreeFormEx : public IUnknown
{
public:
	ICALLTYPE ConstructFilletedLineStrip(double radius, IArcSpline3DCurve** curve) = 0;
	ICALLTYPE ConstructFilletedLineStrip2(IArrayNum* radii, IArcSpline3DCurve** curve) = 0;
	IUNKNOWN_
};

// {4B8EA990-35D5-4130-BB11-8CCF5FCE8F00}
static const GUID IID_ILine = { 0x4b8ea990, 0x35d5, 0x4130, { 0xbb, 0x11, 0x8c, 0xcf, 0x5f, 0xce, 0x8f, 0x0 } };
class ILine : public IUnknown
{
public:
	ICALLTYPE SetOrigin(DIPoint* origin) = 0;
	ICALLTYPE GetOrigin(DIPoint* origin) = 0;
	ICALLTYPE SetDirection(DIVect* direction) = 0;
	ICALLTYPE GetDirection(DIVect* direction) = 0;
	ICALLTYPE GetPoint(double position, DIPoint* point) = 0;
	IUNKNOWN_
};

// {2AFDD2AB-2774-4418-9716-B6E901990CB5}
static const GUID IID_ILineGeom = { 0x2afdd2ab, 0x2774, 0x4418, { 0x97, 0x16, 0xb6, 0xe9, 0x1, 0x99, 0xc, 0xb5 } };
class ILineGeom : public IUnknown
{
public:
	ICALLTYPE RotatePoint(double degrees, DIPoint* point) = 0;
	ICALLTYPE IntersectPlane(DIPoint* locationPlane, DIVect* normalPlane, double* position, int* result) = 0;
	ICALLTYPE GetNearestPoint(DIPoint* point, double* position, double* distance) = 0;
	IUNKNOWN_
};

// {77A549FC-5E39-43C5-A74D-2EA43ADA3FDD}
static const GUID IID_ISurfaceMetric_KC = { 0x77a549fc, 0x5e39, 0x43c5, { 0xa7, 0x4d, 0x2e, 0xa4, 0x3a, 0xda, 0x3f, 0xdd } };
class ISurfaceMetric_KC : public IUnknown
{
public:
	ICALLTYPE ProjectPoint(DIPoint* pt, ISurfacePointKC** iPointProjected) = 0;
	ICALLTYPE CreateProjectionCurve(ICurve* iCurve, ICurve** iCurveProjected) = 0;
	IUNKNOWN_
};

// {B85CD98B-29B1-45c6-843C-BD921504FC09}
static const GUID IID_IMove_KC = { 0xb85cd98b, 0x29b1, 0x45c6, { 0x84, 0x3c, 0xbd, 0x92, 0x15, 0x4, 0xfc, 0x9 } };
class IMove_KC : public IUnknown
{
public:
	ICALLTYPE GetFrameAt(double t, IFrame* frame) = 0;
	ICALLTYPE GetOriginPath(ICurve **curve) = 0;
	ICALLTYPE SetOriginPath(ICurve *curve) = 0;
	ICALLTYPE GetAxisPath(int axis, ICurve **curve) = 0;
	ICALLTYPE SetAxisPath(int axis, ICurve *curve) = 0;
	ICALLTYPE CreateAttributes(int type) = 0;
	ICALLTYPE DefineTangential() = 0;
	IUNKNOWN_
};

// {FE0F1E9B-73DA-4ac5-9A6D-758BEE9A5B94}
static const GUID IID_IDraw = { 0xfe0f1e9b, 0x73da, 0x4ac5, { 0x9a, 0x6d, 0x75, 0x8b, 0xee, 0x9a, 0x5b, 0x94 } };
class IDraw : public IUnknown
{
public:
	ICALLTYPE Begin(EDrawPrimitive mode) = 0;
	ICALLTYPE End() = 0;
	ICALLTYPE Vertex(double x, double y, double z) = 0;
	ICALLTYPE Normal(double nx, double ny, double nz) = 0;
	ICALLTYPE Color(double red, double green, double blue, double alpha) = 0;
	ICALLTYPE LineWidth(float width) = 0;
	ICALLTYPE PointSize(float size) = 0;
	ICALLTYPE PushAttrib() = 0;
	ICALLTYPE PopAttrib() = 0;
	ICALLTYPE PushMatrix() = 0;
	ICALLTYPE PopMatrix() = 0;
	ICALLTYPE Translate(double vx, double vy, double vz) = 0;
	ICALLTYPE Rotate(double angle, double vx, double vy, double vz) = 0;
	ICALLTYPE LoadIdentity() = 0;
	IUNKNOWN_
};

// {F40A7C62-52BC-49b3-9E7F-5F2718BA8792}
static const GUID IID_IDrawUtil = { 0xf40a7c62, 0x52bc, 0x49b3, { 0x9e, 0x7f, 0x5f, 0x27, 0x18, 0xba, 0x87, 0x92 } };
class IDrawUtil : public IUnknown
{
public:
	ICALLTYPE Cylinder(double baseRadius, double topRadius, double height) = 0;
	ICALLTYPE Disk(double innerRadius, double outerRadius) = 0;
	ICALLTYPE Sphere(double radius) = 0;
	ICALLTYPE SetResolution(int resol) = 0;
	ICALLTYPE GetResolution(int* resol) = 0;
	ICALLTYPE Box(double width, double height, double length, int wireframe) = 0;
	ICALLTYPE Rectangle(double width, double height, int wireframe) = 0;
	IUNKNOWN_
};

// {4F4400DB-ABAF-4648-9945-D11228283D04}
static const GUID IID_IDraw2 = { 0x4f4400db, 0xabaf, 0x4648, { 0x99, 0x45, 0xd1, 0x12, 0x28, 0x28, 0x3d, 0x4 } };
class IDraw2 : public IUnknown
{
public:
	ICALLTYPE SetCurrentFrame(IFrame* frame) = 0;
	IUNKNOWN_
};

class IMeshT : public IUnknown
{
public:
	ICALLTYPE GetVertexIterator(IIteratorT** iter) = 0;
	ICALLTYPE GetSimplexIterator(IIteratorT** iter) = 0;
	ICALLTYPE GetVertexCoord(POSNT vertex, double* x, double* y, double* z) = 0;
	ICALLTYPE GetNormal(POSNT vertex, double* vx, double* vy, double* vz) = 0;
	ICALLTYPE GetSimplexVertices(POSNT simplex, POSNT* vert0, POSNT* vert1, POSNT* vert2) = 0;
	ICALLTYPE IsValid(VARIANT_BOOL* ret) = 0;
	IUNKNOWN_
};

class IMeshModsT : public IUnknown
{
public:
	ICALLTYPE SetVertexCoord(POSNT vertex, double x, double y, double z) = 0;
	ICALLTYPE SetNormal(POSNT vertex, double vx, double vy, double vz) = 0;
	ICALLTYPE FixupNormals(double creaseDegrees, double tolerDegrees) = 0;
	ICALLTYPE Offset(double dist) = 0;
	ICALLTYPE Clear() = 0;
	ICALLTYPE ClearList(int dim) = 0;
	ICALLTYPE Begin() = 0;
	ICALLTYPE End(int action) = 0;
	ICALLTYPE SwapSide() = 0;
	ICALLTYPE ToLocal(IFrame* frame) = 0;
	ICALLTYPE ToGlobal(IFrame* frame) = 0;
	IUNKNOWN_
};

class IMeshShading_KCT : public IUnknown
{
public:
	ICALLTYPE SetShadingType(int type) = 0;
	ICALLTYPE GetShadingType(int* type) = 0;
	ICALLTYPE SetVertexColor(POSNT vertex, byte red, byte green, byte blue, byte alpha) = 0;
	ICALLTYPE GetVertexColor(POSNT vertex, byte* red, byte* green, byte* blue, byte* alpha) = 0;
	ICALLTYPE IsShaderInitOK(VARIANT_BOOL* ret) = 0;
	ICALLTYPE DeleteShader() = 0;
	IUNKNOWN_
};

// {F283F885-44AA-4da2-9559-A89092879900}
static const GUID IID_IVertex = { 0xf283f885, 0x44aa, 0x4da2, { 0x95, 0x59, 0xa8, 0x90, 0x92, 0x87, 0x99, 0x0 } };
class IVertex : public IUnknown
{
public:
	ICALLTYPE GetVertexCoord(double* x, double* y, double* z) = 0;
	ICALLTYPE SetVertexCoord(double x, double y, double z) = 0;
	ICALLTYPE GetNormalCount(int* count) = 0;
	ICALLTYPE SetNormalCount(int count) = 0;
	ICALLTYPE GetNormal(int which, double* x, double* y, double* z) = 0;
	ICALLTYPE SetNormal(int which, double vx, double vy, double vz) = 0;
	IUNKNOWN_
};

class ISimplexT : public IUnknown
{
public:
	ICALLTYPE GetVertexCount(int* count) = 0;
	ICALLTYPE GetVertexPosition(int index, POSNT* vertex) = 0;
	ICALLTYPE GetFacetNormal(double* vx, double* vy, double* vz) = 0;
	ICALLTYPE GetNormal(int vertex, double* vx, double* vy, double* vz) = 0;
	ICALLTYPE GetNormalIndex(int vertex, int* index) = 0;
	ICALLTYPE SetNormalIndex(int vertex, int index) = 0;
	IUNKNOWN_
};

class IEdge_KCT : public IUnknown
{
public:
	ICALLTYPE GetEnd(int end, IVertex** vertex) = 0;
	ICALLTYPE GetAdjacentSimplex(int side, ISimplexT** simplex) = 0;
	ICALLTYPE IsInteriorEdge(VARIANT_BOOL* interior) = 0;
	IUNKNOWN_
};

class ISimplexEx_KCT : public IUnknown
{
public:
	ICALLTYPE GetEdge(int side, IEdge_KCT** edge) = 0;
	ICALLTYPE GetAdjacentSimplex(int side, ISimplexEx_KCT** simplex) = 0;
	ICALLTYPE IsInteriorSimplex(VARIANT_BOOL* interior) = 0;
	IUNKNOWN_
};

// {9EBA7404-6CA6-466c-8801-F61DC8EF01AA}
static const GUID IID_IMeshBuilder_KC = { 0x9eba7404, 0x6ca6, 0x466c, { 0x88, 0x1, 0xf6, 0x1d, 0xc8, 0xef, 0x1, 0xaa } };
class IMeshBuilder_KC : public IUnknown
{
public:
	ICALLTYPE SetCurrentNormal(double vx, double vy, double vz) = 0;
	ICALLTYPE SetCurrentColor(float red, float green, float blue, float alpha) = 0;
	ICALLTYPE AddVertex(double x, double y, double z, int* indxOut) = 0;
	ICALLTYPE AddSimplex(int i0, int i1, int i2) = 0;
	ICALLTYPE AddQuad(int i0, int i1, int i2, int i3) = 0;
	ICALLTYPE Reset() = 0;
	IUNKNOWN_
};

// {AB539D40-109D-4EEB-8E20-97BE254A5B66}
static const GUID IID_ISelection_KC = { 0xab539d40, 0x109d, 0x4eeb, { 0x8e, 0x20, 0x97, 0xbe, 0x25, 0x4a, 0x5b, 0x66 } };
class ISelection_KC : public IUnknown
{
public:
	ICALLTYPE GetSelectedItems(IArray* items) = 0;
	ICALLTYPE SetSelected(IObject_KC* item, VARIANT_BOOL selected) = 0;
	ICALLTYPE ClearSelection() = 0;
	ICALLTYPE CloseContext() = 0;
	IUNKNOWN_
};

class IVertexEx_KCT : public IUnknown
{
public:
	ICALLTYPE GetStarElementCount(int elementType, int* count) = 0;
	ICALLTYPE GetStarVertex(int i, IVertexEx_KCT** vertex) = 0;
	ICALLTYPE GetStarEdge(int i, IEdge_KCT** edge) = 0;
	ICALLTYPE GetStarSimplex(int i, ISimplexEx_KCT** simplex) = 0;
	ICALLTYPE IsInteriorVertex(VARIANT_BOOL* interior) = 0;
	ICALLTYPE GetLinkingEdge(IVertex* vertex, IEdge_KCT** edge) = 0;
	ICALLTYPE FindStarElement(IObject_KC* vertexSimplexOrEdge, int* index) = 0;
	IUNKNOWN_
};

// {C765B1FC-CB56-4c37-BADB-C8A9A90A05B9}
static const GUID IID_IRefMap_KC = { 0xc765b1fc, 0xcb56, 0x4c37, { 0xba, 0xdb, 0xc8, 0xa9, 0xa9, 0xa, 0x5, 0xb9 } };
class IRefMap_KC : public IUnknown
{
public:
	ICALLTYPE GetCount(int* count) = 0;
	ICALLTYPE SetAt(POSNT key, POSNT value) = 0;
	ICALLTYPE Lookup(POSNT key, POSNT* value, VARIANT_BOOL* found) = 0;
	ICALLTYPE RemoveKey(POSNT key, VARIANT_BOOL* found) = 0;
	ICALLTYPE RemoveAll() = 0;
	ICALLTYPE GetStartPosition(POSNT* position) = 0;
	ICALLTYPE GetNext(POSNT* position, POSNT* key, POSNT* value) = 0;
	IUNKNOWN_
};

class IMeshExT : public IUnknown
{
public:
	ICALLTYPE GetVertex(POSNT vertex, IVertex** iVertex) = 0;
	ICALLTYPE GetSimplex(POSNT simplex, ISimplexT** iSimplex) = 0;
	ICALLTYPE GetVertexPosition(IVertex* iVertex, POSNT* position) = 0;
	ICALLTYPE GetSimplexPosition(ISimplexT* simplex, POSNT* position) = 0;
	ICALLTYPE Upgrade() = 0;
	ICALLTYPE IsUpgraded(VARIANT_BOOL* upgraded) = 0;
	ICALLTYPE Downgrade() = 0;
	ICALLTYPE IsClosedSurface(VARIANT_BOOL* closed) = 0;
	ICALLTYPE Extract(IObjectOriented* proximity, int elements, VARIANT_BOOL move, VARIANT_BOOL replicate, IMeshT* iMeshExtracted, IRefMap_KC** map) = 0;
	ICALLTYPE Extract2(IFrame* frameOfThis, IObjectOriented* proximity, int elements, VARIANT_BOOL move, VARIANT_BOOL replicate, IMeshT* iMeshExtracted, IRefMap_KC** map) = 0;
	ICALLTYPE Merge(IMeshT* iMesh, int elements, VARIANT_BOOL move, VARIANT_BOOL replicate, IRefMap_KC** map) = 0;
	ICALLTYPE SplitLinkedComponents(IArray2** components, IRefMap_KC** map, VARIANT_BOOL* found) = 0;
	ICALLTYPE Refine(EMeshRefineType type) = 0;
	ICALLTYPE SmoothLaplacian(int steps) = 0;
	IUNKNOWN_
};

class IMeshTopolT : public IUnknown
{
public:
	ICALLTYPE AddVertex(POSNT* posnResult) = 0;
	ICALLTYPE RemoveVertex(POSNT vertex) = 0;
	ICALLTYPE AddSimplex(POSNT vert0, POSNT vert1, POSNT vert2, double creaseAngle) = 0;
	ICALLTYPE RemoveSimplex(POSNT simplex) = 0;
	IUNKNOWN_
};

class IMeshAlgorT : public IUnknown
{
public:
	ICALLTYPE AddVertex(double x, double y, double z, POSNT* posnResult) = 0;
	ICALLTYPE BuildMesh(float Radius, float minDist, float creaseDegrees) = 0;
	ICALLTYPE GetDefaults(float* Radius, float* minDist, float* creaseDegrees) = 0;
	IUNKNOWN_
};

// {7608F844-1085-4208-ABCB-BAE36AF4DDB1}
static const GUID IID_IDataContext = { 0x7608f844, 0x1085, 0x4208, { 0xab, 0xcb, 0xba, 0xe3, 0x6a, 0xf4, 0xdd, 0xb1 } };
class IDataContext : public IUnknown
{
public:
	ICALLTYPE SetVerbose(VARIANT_BOOL verbose) = 0;
	ICALLTYPE SetNoWarnings(VARIANT_BOOL noWarnings) = 0;
	IUNKNOWN_
};

// {7126A905-D9D9-42a9-8E95-6E046EB87E70}
static const GUID IID_ISTLDataContext = { 0x7126a905, 0xd9d9, 0x42a9, { 0x8e, 0x95, 0x6e, 0x4, 0x6e, 0xb8, 0x7e, 0x70 } };
class ISTLDataContext : public IDataContext
{
public:
	ICALLTYPE SetBinary(VARIANT_BOOL binary) = 0;
	ICALLTYPE SetSeparateObjects(VARIANT_BOOL separate) = 0;
	ICALLTYPE SetCreaseAngle(double degrees) = 0;
	ICALLTYPE SetBuildTree(VARIANT_BOOL build) = 0;
	ICALLTYPE SetAnalysisType(int type) = 0;
	IUNKNOWN_
};

// {7588C295-C914-4B4B-8D28-8A095D25E17F}
static const GUID IID_IFormatVersionDataContext = { 0x7588c295, 0xc914, 0x4b4b, { 0x8d, 0x28, 0x8a, 0x9, 0x5d, 0x25, 0xe1, 0x7f } };
class IFormatVersionDataContext : public IDataContext
{
public:
	ICALLTYPE SetSaveFormatVersion(int version) = 0;
	ICALLTYPE GetSaveFormatVersion(int* version) = 0;
	IUNKNOWN_
};

// {5BD94CBB-BD54-44bb-B384-4440836D4665}
static const GUID IID_IModelEx = { 0x5bd94cbb, 0xbd54, 0x44bb, { 0xb3, 0x84, 0x44, 0x40, 0x83, 0x6d, 0x46, 0x65 } };
class IModelEx : public IUnknown
{
public:
	ICALLTYPE Load(char* szFilePath) = 0;
	ICALLTYPE Save(char* szFilePath) = 0;
	ICALLTYPE LoadEx(char* szFilePath, IDataContext* options) = 0;
	ICALLTYPE SaveEx(char* szFilePath, IDataContext* options) = 0;
	ICALLTYPE LoadFromMemory(SAFEARRAY **data, int format) = 0;
	ICALLTYPE SaveToMemory(SAFEARRAY **data, int format) = 0;// format is member of ECADFormat enum. See DITypes.h
	ICALLTYPE SetPersistModified(VARIANT_BOOL modified) = 0;
	ICALLTYPE IsPersistModified(VARIANT_BOOL* modified) = 0;
	IUNKNOWN_
};

// {4A45F380-6B5B-4A18-8698-C5FAC93C2F25}
static const GUID IID_ICADFormat_KC = { 0x4a45f380, 0x6b5b, 0x4a18, { 0x86, 0x98, 0xc5, 0xfa, 0xc9, 0x3c, 0x2f, 0x25 } };
class ICADFormat_KC : public IUnknown
{
public:
	ICALLTYPE GetFormatID(ECADFormat* format) = 0;
	ICALLTYPE GetVersionCount(int* count) = 0;
	ICALLTYPE GetVersion(int index, int* version) = 0;
	ICALLTYPE GetVersionString(int version, BSTR *pbstrDescription) = 0;
	ICALLTYPE FindVersion(int version, int* index) = 0;
	ICALLTYPE SetDeafultSaveVersionIndex(int index) = 0;
	ICALLTYPE GetDeafultSaveVersionIndex(int* index) = 0;
	ICALLTYPE GetExtension(VARIANT_BOOL includeDot, BSTR* extension) = 0;
	IUNKNOWN_
};

// {A006385D-67DA-4091-8511-1EB879798C31}
static const GUID IID_IDataConvert_KC = { 0xa006385d, 0x67da, 0x4091, { 0x85, 0x11, 0x1e, 0xb8, 0x79, 0x79, 0x8c, 0x31 } };
class IDataConvert_KC : public IUnknown
{
public:
	ICALLTYPE Convert(char* inFile, char* outFile, int formatVersion) = 0;
	ICALLTYPE ConvertMultiple(char* wildcard, char* outFolder, ECADFormat formatConvertTo, int formatVersion) = 0;
	ICALLTYPE DetectFormat(char* file, ECADFormat* format) = 0;
	ICALLTYPE GetFormat(ECADFormat formatID, ICADFormat_KC** iFormat) = 0;
	IUNKNOWN_
};

// {4968C48B-2DBE-4111-81C7-6CD0C83DE3F0}
static const GUID IID_IText = { 0x4968c48b, 0x2dbe, 0x4111, { 0x81, 0xc7, 0x6c, 0xd0, 0xc8, 0x3d, 0xe3, 0xf0 } };
class IText : public IUnknown
{
public:
	ICALLTYPE SetText(char* text) = 0;
	ICALLTYPE GetText(BSTR *text) = 0;
	IUNKNOWN_
};

// {30471922-74DA-4671-8B3C-EE1537F06F98}
static const GUID IID_IDIFont = { 0x30471922, 0x74da, 0x4671, { 0x8b, 0x3c, 0xee, 0x15, 0x37, 0xf0, 0x6f, 0x98 } };
class IDIFont : public IUnknown
{
public:
	ICALLTYPE SetSize(int size) = 0;
	ICALLTYPE GetSize(int* size) = 0;
	ICALLTYPE SetFaceName(char* name) = 0;
	ICALLTYPE GetFaceName(BSTR* name) = 0;
	ICALLTYPE GetColor(float * pfRed, float * pfGreen, float * pfBlue) = 0;
	ICALLTYPE SetColor(float fRed, float fGreen, float fBlue) = 0;
	ICALLTYPE GetTextBackgroundColor(float * pfRed, float * pfGreen, float * pfBlue) = 0;
	ICALLTYPE SetTextBackgroundColor(float fRed, float fGreen, float fBlue) = 0;
	IUNKNOWN_
};

// {88AA6EBD-3409-445d-8E0B-BCD895A30411}
static const GUID IID_IBoolSection = { 0x88aa6ebd, 0x3409, 0x445d, { 0x8e, 0xb, 0xbc, 0xd8, 0x95, 0xa3, 0x4, 0x11 } };
class IBoolSection : public IUnknown
{
public:
	ICALLTYPE Subtract(ISection* object) = 0;
	ICALLTYPE Unite(ISection* object) = 0;
	ICALLTYPE Intersect(ISection* object) = 0;
	IUNKNOWN_
};

// {CCAD0E1B-B7D4-4a8c-8549-D2E736DFBDF8}
static const GUID IID_IBoolSectionEx = { 0xccad0e1b, 0xb7d4, 0x4a8c, { 0x85, 0x49, 0xd2, 0xe7, 0x36, 0xdf, 0xbd, 0xf8 } };
class IBoolSectionEx : public IUnknown
{
public:
	ICALLTYPE Execute(EBooleanOperation operation, ISection* object, VARIANT_BOOL createIntersLine, IArray** edges) = 0;
	ICALLTYPE Execute2(ISection* object, EBoolOpContext* context) = 0;
	ICALLTYPE CreateExtendedContext(IDictionary_KC** contextExt) = 0;
	ICALLTYPE Execute3(EBooleanOperation operation, ISection* object, IDictionary_KC* contextExt) = 0;
	ICALLTYPE Clip(IObjectOriented* objectClipper, VARIANT_BOOL needTopLinkedComponentOnly) = 0;
	ICALLTYPE Clip2(IObjectOriented* objectClipper, IListT** listISection) = 0;
	IUNKNOWN_
};

// {CF084FEA-DE4B-4ec9-918C-1C36D4512DC7}
static const GUID IID_IMatrixData = { 0xcf084fea, 0xde4b, 0x4ec9, { 0x91, 0x8c, 0x1c, 0x36, 0xd4, 0x51, 0x2d, 0xc7 } };
class IMatrixData : public IUnknown
{
public:
	ICALLTYPE Get(int row, int column, double* val) = 0;
	ICALLTYPE Set(int row, int column, double val) = 0;
	ICALLTYPE GetTriple(int row, double* v0, double* v1, double* v2) = 0;
	ICALLTYPE SetTriple(int row, double v0, double v1, double v2) = 0;
	IUNKNOWN_
};

// {ADA0FA17-BCF6-4812-8A23-EB2B88556CB3}
static const GUID IID_IPipeSection = { 0xada0fa17, 0xbcf6, 0x4812, { 0x8a, 0x23, 0xeb, 0x2b, 0x88, 0x55, 0x6c, 0xb3 } };
class IPipeSection : public IUnknown
{
public:
	ICALLTYPE GetCentralCurve(ICurve **curve) = 0;
	ICALLTYPE SetCentralCurve(ICurve *curve) = 0;
	ICALLTYPE GetDiameter(double* diameter) = 0;
	ICALLTYPE SetDiameter(double diameter) = 0;
	ICALLTYPE GetInnerDiameter(double* diam) = 0;
	ICALLTYPE SetInnerDiameter(double diam) = 0;
	ICALLTYPE GetInternalSurfaceFlag(VARIANT_BOOL* internSurf) = 0;
	ICALLTYPE SetInternalSurfaceFlag(VARIANT_BOOL internSurf) = 0;
	ICALLTYPE GetRenderType(int* type) = 0;
	ICALLTYPE SetRenderType(int type) = 0;
	IUNKNOWN_
};

// {FB0001B1-B1F6-48b5-9562-E955F21B1D4A}
static const GUID IID_ILineSection_KC = { 0xfb0001b1, 0xb1f6, 0x48b5, { 0x95, 0x62, 0xe9, 0x55, 0xf2, 0x1b, 0x1d, 0x4a } };
class ILineSection_KC : public IUnknown
{
public:
	ICALLTYPE GetCurve(ICurve **curve) = 0;
	ICALLTYPE SetCurve(ICurve *curve) = 0;
	IUNKNOWN_
};

// {7FF6A85F-0E18-44bb-8762-22D137EAA9A3}
static const GUID IID_ISectionPointSet = { 0x7ff6a85f, 0xe18, 0x44bb, { 0x87, 0x62, 0x22, 0xd1, 0x37, 0xea, 0xa9, 0xa3 } };
class ISectionPointSet : public IUnknown
{
public:
	ICALLTYPE GetPoints(IMatrixData** points) = 0;
	ICALLTYPE GetPointSize(int* size) = 0;
	ICALLTYPE PointSize(int size) = 0;
	ICALLTYPE GetColorPerVertex(VARIANT_BOOL *val) = 0;
	ICALLTYPE ColorPerVertex(VARIANT_BOOL val) = 0;
	ICALLTYPE GetColors(IMatrixData** colors) = 0;
	ICALLTYPE GetPointCount(int* count) = 0;
	IUNKNOWN_
};

// Objects implementing this interface are inherited from Point Set type and are using the underlying set of points
// as ends of segments. The index is an array of integers - indecies of points in the base array.
// The Line Strip can consists of several connected strips. Strips are separated by -1 in the index.
// For example index (0,1,2,-1,3,4) means that there are two strips in the section. The first one consists of
// points 0,1,2 and the second one of points 3 and 4.
// {D87DDAC1-6E93-45a0-AF51-073CEBD7C857}
static const GUID IID_ISectionLineStrip = { 0xd87ddac1, 0x6e93, 0x45a0, { 0xaf, 0x51, 0x7, 0x3c, 0xeb, 0xd7, 0xc8, 0x57 } };
class ISectionLineStrip : public IUnknown
{
public:
	ICALLTYPE GetIndex(IArray2** index) = 0;
	ICALLTYPE GetLineWidth(int* width) = 0;
	ICALLTYPE LineWidth(int width) = 0;
	ICALLTYPE GetStippleFlag(int* stipple) = 0;
	ICALLTYPE SetStippleFlag(int stipple) = 0;
	ICALLTYPE GetStipplePattern(int* pattern) = 0;
	ICALLTYPE SetStipplePattern(int pattern) = 0;
	ICALLTYPE GetStippleFactor(int* factor) = 0;
	ICALLTYPE SetStippleFactor(int factor) = 0;
	IUNKNOWN_
};

// {5F4341D3-F989-4170-8BBE-C8D4B99CD6AC}
static const GUID IID_ILineStyle_KC = { 0x5f4341d3, 0xf989, 0x4170, { 0x8b, 0xbe, 0xc8, 0xd4, 0xb9, 0x9c, 0xd6, 0xac } };
class ILineStyle_KC : public IUnknown
{
public:
	ICALLTYPE SetColor(Color_KC* color) = 0;
	ICALLTYPE GetColor(Color_KC* color) = 0;
	ICALLTYPE GetLineWidth(float* width) = 0;
	ICALLTYPE SetLineWidth(float width) = 0;
	ICALLTYPE GetStippleFlag(VARIANT_BOOL* stipple) = 0;
	ICALLTYPE SetStippleFlag(VARIANT_BOOL stipple) = 0;
	ICALLTYPE GetStipplePattern(int* pattern) = 0;
	ICALLTYPE SetStipplePattern(int pattern) = 0;
	ICALLTYPE GetStippleFactor(int* factor) = 0;
	ICALLTYPE SetStippleFactor(int factor) = 0;
	IUNKNOWN_
};

// {B55E80ED-A2CF-45fa-832B-4D934B8581FD}
static const GUID IID_IRange = { 0xb55e80ed, 0xa2cf, 0x45fa, { 0x83, 0x2b, 0x4d, 0x93, 0x4b, 0x85, 0x81, 0xfd } };
class IRange : public IUnknown
{
public:
	ICALLTYPE GetMin(double* min) = 0;
	ICALLTYPE GetMax(double* max) = 0;
	ICALLTYPE Set(double min, double max) = 0;
	IUNKNOWN_
};

// {DB0199C1-E348-4ebe-9AEC-1C17FAFEA8EA}
static const GUID IID_IColor = { 0xdb0199c1, 0xe348, 0x4ebe, { 0x9a, 0xec, 0x1c, 0x17, 0xfa, 0xfe, 0xa8, 0xea } };
class IColor : public IUnknown
{
public:
	ICALLTYPE Get(float* red, float* green, float* blue, float* alpha) = 0;
	ICALLTYPE Set(float red, float green, float blue, float alpha) = 0;
	IUNKNOWN_
};

// {A9BF85A3-71A3-4d44-A8CF-3B7634A9BFC6}
static const GUID IID_IViewPoint = { 0xa9bf85a3, 0x71a3, 0x4d44, { 0xa8, 0xcf, 0x3b, 0x76, 0x34, 0xa9, 0xbf, 0xc6 } };
class IViewPoint : public IUnknown
{
public:
	ICALLTYPE Store() = 0;
	ICALLTYPE SetCurrent() = 0;
	ICALLTYPE GetViewVolumeDimensions(double* sx, double* sy, double* sz) = 0;
	ICALLTYPE SetViewVolumeDimensions(double sx, double sy) = 0;
	ICALLTYPE SetCurrent2(int transitionMilliseconds) = 0;
	IUNKNOWN_
};

// {FC055F16-256F-455e-91D6-021BF8D635F0}
static const GUID IID_IViewPointArray = { 0xfc055f16, 0x256f, 0x455e, { 0x91, 0xd6, 0x2, 0x1b, 0xf8, 0xd6, 0x35, 0xf0 } };
class IViewPointArray : public IUnknown
{
public:
	ICALLTYPE GetCount(int* count) = 0;
	ICALLTYPE GetAt(int index, IViewPoint** vp) = 0;
	ICALLTYPE RemoveAt(int index) = 0;
	ICALLTYPE Add(char* name) = 0;
	ICALLTYPE Default(int index) = 0;
	ICALLTYPE GetDefault(int* index) = 0;
	IUNKNOWN_
};

// {DC05B0C6-D2EF-4e0a-B7DE-91C80A61B5E7}
static const GUID IID_ILookAt = { 0xdc05b0c6, 0xd2ef, 0x4e0a, { 0xb7, 0xde, 0x91, 0xc8, 0xa, 0x61, 0xb5, 0xe7 } };
class ILookAt : public IUnknown
{
public:
	ICALLTYPE LookAt(DIPoint* point, DIVect* dirUp) = 0;
	ICALLTYPE LookAtItem(IItem* iItem, DIVect* dirUp) = 0;
	ICALLTYPE LookAtFrame(I3DObject* i3DObject, DIVect* dirUp) = 0;
	ICALLTYPE LookAtModel(IModel* iModel, DIVect* dirUp) = 0;
	IUNKNOWN_
};

// {B43E05CD-8C3F-4d11-B75A-217522CBE50B}
static const GUID IID_IStdShape = { 0xb43e05cd, 0x8c3f, 0x4d11, { 0xb7, 0x5a, 0x21, 0x75, 0x22, 0xcb, 0xe5, 0xb } };
class IStdShape : public IUnknown
{
public:
	ICALLTYPE Cylinder(double baseRadius, double topRadius, double height, ISection** cylinder) = 0;
	ICALLTYPE Disk(double innerRadius, double outerRadius, ISection** disk) = 0;
	ICALLTYPE Sphere(double radius, ISection** sphere) = 0;
	ICALLTYPE Box(double width, double height, double length, ISection** box) = 0;
	ICALLTYPE Torus(double radius, double radiusTube, ISection** torus) = 0;
	ICALLTYPE Triangle(DIPoint* pt0, DIPoint* pt1, DIPoint* pt2, ISection** obj) = 0;
	ICALLTYPE Polygon(DIPoint* pt0, DIPoint* pt1, DIPoint* pt2, DIPoint* pt3, ISection** obj) = 0;
	ICALLTYPE Line(DIPoint* pt0, DIPoint* pt1, ISection** obj) = 0;
	ICALLTYPE Circle(double radius, ISection** obj) = 0;
	ICALLTYPE Rectangle(double width, double height, ISection** obj) = 0;
	ICALLTYPE LineInf(DIPoint* origin, DIVect* direction, VARIANT_BOOL ray, ISection** obj) = 0;
	ICALLTYPE Ellipse(double a, double b, ISection** obj) = 0;
	IUNKNOWN_
};

// {32AB9803-326C-4ed9-A5A5-34DAEDFBAFBF}
static const GUID IID_ITransform = { 0x32ab9803, 0x326c, 0x4ed9, { 0xa5, 0xa5, 0x34, 0xda, 0xed, 0xfb, 0xaf, 0xbf } };
class ITransform : public IUnknown
{
public:
	ICALLTYPE TranslateLoc(double vx, double vy, double vz, VARIANT_BOOL includeChildren) = 0;
	ICALLTYPE ScaleLoc(double scalex, double scaley, double scalez, VARIANT_BOOL includeChildren) = 0;
	ICALLTYPE RotateLoc(double degrees, DIVect* axis, VARIANT_BOOL frameOrObject) = 0;
	ICALLTYPE Apply(IFrame* target, VARIANT_BOOL frameOrObject) = 0;
	ICALLTYPE RotateLoc2(double degrees, DIPoint* originAxis, DIVect* directionAxis, VARIANT_BOOL frameOrObject, VARIANT_BOOL includeChildren) = 0;
	IUNKNOWN_
};

// {901FA111-03B3-4b52-B1AD-10949302EAFC}
static const GUID IID_IFrameInPlace_KC = { 0x901fa111, 0x3b3, 0x4b52, { 0xb1, 0xad, 0x10, 0x94, 0x93, 0x2, 0xea, 0xfc } };
class IFrameInPlace_KC : public IUnknown
{
public:
	ICALLTYPE Translate(DIVect* translationVector, VARIANT_BOOL globalAxes, VARIANT_BOOL includeChildren) = 0;
	ICALLTYPE Rotate(double degrees, RayKC* axis, VARIANT_BOOL globalAxes, VARIANT_BOOL includeChildren) = 0;
	ICALLTYPE Apply(IFrame* target) = 0;
	IUNKNOWN_
};

class I3DBuggerT : public IUnknown
{
public:
	ICALLTYPE DumpObject(IUnknown* representation, POSNT objectHash, VARIANT_BOOL writeThrough, VARIANT_BOOL transient) = 0;
	ICALLTYPE Dump() = 0;
	ICALLTYPE AttachStream(char* name) = 0;
	ICALLTYPE CloseStream(char* name) = 0;
	ICALLTYPE BeginSession() = 0;
	ICALLTYPE BeginDump() = 0;
	ICALLTYPE BlockAllDumps(VARIANT_BOOL block) = 0;
	IUNKNOWN_
};

class ICollisionDetectorT : public IUnknown
{
public:
	ICALLTYPE AttachComponent(IUnknown* component) = 0;
	ICALLTYPE AddObject(ISection* object, VARIANT_BOOL stationary, VARIANT_BOOL includeChildren) = 0;
	ICALLTYPE Start(double minDist) = 0;
	ICALLTYPE Stop() = 0;
	ICALLTYPE GetCollisions(IRelationT** nearest, double* dist) = 0;
	ICALLTYPE ClearObjectArray() = 0;
	ICALLTYPE AttachComponent2(VARIANT* component) = 0;
	IUNKNOWN_
};

class IKCPathCollisionDetectorT : public IUnknown
{
public:
	ICALLTYPE Detect(PathDetectorContext* context, double* tCollision) = 0;
	ICALLTYPE GetCollidedObjects(IMetricsT** object0, IMetricsT** object1) = 0;
	IUNKNOWN_
};

// {EC1DE31C-1F1B-4022-A0E3-7ECFDA2231BA}
static const GUID IID_IBlob = { 0xec1de31c, 0x1f1b, 0x4022, { 0xa0, 0xe3, 0x7e, 0xcf, 0xda, 0x22, 0x31, 0xba } };
class IBlob : public IUnknown
{
public:
	ICALLTYPE GetSize(int* bytes) = 0;
	ICALLTYPE Load(char* szFilePath) = 0;
	ICALLTYPE Save(char* szFilePath) = 0;
	ICALLTYPE Copy(SAFEARRAY **data) = 0;
	ICALLTYPE CopyTo(SAFEARRAY **data) = 0;
	IUNKNOWN_
};

// {9D859998-B666-4aaf-8579-003C155FCCEF}
static const GUID IID_IBitmap = { 0x9d859998, 0xb666, 0x4aaf, { 0x85, 0x79, 0x0, 0x3c, 0x15, 0x5f, 0xcc, 0xef } };
class IBitmap : public IUnknown
{
public:
	ICALLTYPE GetBitmapData(IBlob** fileData) = 0;
	ICALLTYPE GetInfo(KCBitmap* info) = 0;
	ICALLTYPE GetBits(IBlob** image) = 0;
	IUNKNOWN_
};

// {FEF5B24C-4C45-40a3-B4B9-8E68D2971FC7}
static const GUID IID_IVirtualView = { 0xfef5b24c, 0x4c45, 0x40a3, { 0xb4, 0xb9, 0x8e, 0x68, 0xd2, 0x97, 0x1f, 0xc7 } };
class IVirtualView : public IUnknown
{
public:
	ICALLTYPE Create(PairInt* size, IBitmap** buffer) = 0;
	ICALLTYPE Render() = 0;
	ICALLTYPE Destroy() = 0;
	ICALLTYPE CreateModelView(IModel* model, PairInt* size, IBitmap** buffer) = 0;
	IUNKNOWN_
};

class I3DGridT : public IUnknown
{
public:
	ICALLTYPE Create(int cellsMaxDimension) = 0;
	ICALLTYPE GetCellCount(KCN3* cellCount) = 0;
	ICALLTYPE GetRanges(KCBox* ranges) = 0;
	ICALLTYPE GetBox(KCN3* index, KCBox* ranges) = 0;
	ICALLTYPE FindBox(DIPoint* point, KCN3* index) = 0;
	ICALLTYPE GetCover(VARIANT_BOOL intern, VARIANT_BOOL shell, VARIANT_BOOL external, KCN3* count, IListT** cover) = 0;
	IUNKNOWN_
};

// {96A0E137-0FF9-4893-A76D-826715B05D34}
static const GUID IID_ISurfaceDiff = { 0x96a0e137, 0xff9, 0x4893, { 0xa7, 0x6d, 0x82, 0x67, 0x15, 0xb0, 0x5d, 0x34 } };
class ISurfaceDiff : public IUnknown
{
public:
	ICALLTYPE GetLocalParametrization(DIPoint* pointNearSurface, IKO_Geom_BSplineSurface** proximity) = 0;
	IUNKNOWN_
};

class ISurfaceT : public IUnknown
{
public:
	ICALLTYPE GetPatches(IListT** patches) = 0;
	ICALLTYPE GetEdges(IListT** edges) = 0;
	ICALLTYPE GetVertices(IListT** vertices) = 0;
	IUNKNOWN_
};

// {D374AD6D-03A1-4224-881F-8C21D272E77F}
static const GUID IID_IPatch = { 0xd374ad6d, 0x3a1, 0x4224, { 0x88, 0x1f, 0x8c, 0x21, 0xd2, 0x72, 0xe7, 0x7f } };
class IPatch : public IUnknown
{
public:
	ICALLTYPE GetEdgeComponents(IArray2** edges) = 0;
	IUNKNOWN_
};

// {4D6A7119-13CF-4d2d-8280-9A1C2D99E619}
static const GUID IID_IKCLine = { 0x4d6a7119, 0x13cf, 0x4d2d, { 0x82, 0x80, 0x9a, 0x1c, 0x2d, 0x99, 0xe6, 0x19 } };
class IKCLine : public IUnknown
{
public:
	ICALLTYPE GetEnd(int end, IVertex** vertex) = 0;
	ICALLTYPE IsClosed(VARIANT_BOOL* closed) = 0;
	IUNKNOWN_
};

// {AA5EBDCD-D212-419c-9E25-50658AD33464}
static const GUID IID_IGraphicItem = { 0xaa5ebdcd, 0xd212, 0x419c, { 0x9e, 0x25, 0x50, 0x65, 0x8a, 0xd3, 0x34, 0x64 } };
class IGraphicItem : public IUnknown
{
public:
	ICALLTYPE Draw() = 0;
	ICALLTYPE DrawEx(RenderContext* context) = 0;
	ICALLTYPE AttachView(IView* view) = 0;
	IUNKNOWN_
};

// {90072361-7E9F-454c-961C-75678D5F0DB8}
static const GUID IID_IKCContext = { 0x90072361, 0x7e9f, 0x454c, { 0x96, 0x1c, 0x75, 0x67, 0x8d, 0x5f, 0xd, 0xb8 } };
class IKCContext : public IUnknown
{
public:
	ICALLTYPE GetParamCount(int type, int* count) = 0;
	ICALLTYPE GetIntParam(int id, int *val) = 0;
	ICALLTYPE SetIntParam(int id, int val) = 0;
	ICALLTYPE GetDoubleParam(int id, double *val) = 0;
	ICALLTYPE SetDoubleParam(int id, double val) = 0;
	ICALLTYPE GetBoolParam(int id, VARIANT_BOOL *val) = 0;
	ICALLTYPE SetBoolParam(int id, VARIANT_BOOL val) = 0;
	ICALLTYPE GetStringParam(int id, BSTR *val) = 0;
	ICALLTYPE SetStringParam(int id, char *val) = 0;
	ICALLTYPE GetParam(int id, VARIANT *param) = 0;
	ICALLTYPE SetParam(int id, VARIANT *param) = 0;
	IUNKNOWN_
};

// {D5E4CE6A-83B2-4406-B9C3-3AD01B3311C1}
static const GUID IID_IKCContext2 = { 0xd5e4ce6a, 0x83b2, 0x4406, { 0xb9, 0xc3, 0x3a, 0xd0, 0x1b, 0x33, 0x11, 0xc1 } };
class IKCContext2 : public IUnknown
{
public:
	ICALLTYPE GetColorParam(int id, float* red, float* green, float* blue, float* alpha) = 0;
	ICALLTYPE SetColorParam(int id, float red, float green, float blue, float alpha) = 0;
	IUNKNOWN_
};

// {D3E2ED43-99C3-4935-BA83-6BF9083972FF}
static const GUID IID_IKCStack = { 0xd3e2ed43, 0x99c3, 0x4935, { 0xba, 0x83, 0x6b, 0xf9, 0x8, 0x39, 0x72, 0xff } };
class IKCStack : public IUnknown
{
public:
	ICALLTYPE Push(int attributes) = 0;
	ICALLTYPE Pop(int attributes) = 0;
	IUNKNOWN_
};

// {99691C4D-D6E6-4f55-B9E6-B02FC4D05284}
static const GUID IID_ICircle3D_KC2 = { 0x99691c4d, 0xd6e6, 0x4f55, { 0xb9, 0xe6, 0xb0, 0x2f, 0xc4, 0xd0, 0x52, 0x84 } };
class ICircle3D_KC2 : public IUnknown
{
public:
	ICALLTYPE  GetRadius(double* radius) = 0;
	ICALLTYPE  SetRadius(double radius) = 0;
	ICALLTYPE  GetDisk(VARIANT_BOOL* disk) = 0;
	ICALLTYPE  SetDisk(VARIANT_BOOL disk) = 0;
	IUNKNOWN_
};

// {D123B939-3078-4e7c-B200-398AE4015020}
static const GUID IID_IEllipse_KC = { 0xd123b939, 0x3078, 0x4e7c, { 0xb2, 0x0, 0x39, 0x8a, 0xe4, 0x1, 0x50, 0x20 } };
class IEllipse_KC : public IUnknown
{
public:
	ICALLTYPE  GetA(double* a) = 0;
	ICALLTYPE  SetA(double a) = 0;
	ICALLTYPE  GetB(double* b) = 0;
	ICALLTYPE  SetB(double b) = 0;
	ICALLTYPE  GetPointAtAngle(double angle, double* x, double* y) = 0;
	ICALLTYPE  GetNormalAtAngle(double angle, double* nx, double* ny) = 0;
	IUNKNOWN_
};

// {43F27117-E934-49df-A920-2F778FB1C83E}
static const GUID IID_ITypeBasic_KC = { 0x43f27117, 0xe934, 0x49df, { 0xa9, 0x20, 0x2f, 0x77, 0x8f, 0xb1, 0xc8, 0x3e } };
class ITypeBasic_KC : public IUnknown
{
public:
	ICALLTYPE  GetTypeID(int* tid) = 0;
	ICALLTYPE  GetTypeName(int typeID, BSTR *name) = 0;
	ICALLTYPE  GetCompatibleTypes(int typeID, IArrayNum** types) = 0;
	IUNKNOWN_
};

// {F606034E-4924-40e2-B31F-407D8261FD10}
static const GUID IID_I2DView_KC = { 0xf606034e, 0x4924, 0x40e2, { 0xb3, 0x1f, 0x40, 0x7d, 0x82, 0x61, 0xfd, 0x10 } };
class I2DView_KC : public IUnknown
{
public:
	ICALLTYPE GetContext(IKCContext **context) = 0;
	ICALLTYPE AddText(int xLoc, int yLoc, char* text, IText **iText) = 0;
	ICALLTYPE DeviceToGeom(int k, int coord, double *val) = 0;
	ICALLTYPE GeomToDevice(double val, int coord, int *k) = 0;
	ICALLTYPE RemoveAll() = 0;
	ICALLTYPE AddText2(double x, double y, char* text, IText **iText) = 0;
	IUNKNOWN_
};

// {E52318CC-D4E3-4cbe-85CF-0EB4E5F5165D}
static const GUID IID_IPrizmInfinite_KC = { 0xe52318cc, 0xd4e3, 0x4cbe, { 0x85, 0xcf, 0xe, 0xb4, 0xe5, 0xf5, 0x16, 0x5d } };
class IPrizmInfinite_KC : public IUnknown
{
public:
	ICALLTYPE GetBase(IStrip** strip) = 0;
	ICALLTYPE GetFrame(IFrame3** frame) = 0;
	IUNKNOWN_
};

// {F3EAC606-9797-4831-A8E8-A395EB1FB871}
static const GUID IID_I2DEditor_KC = { 0xf3eac606, 0x9797, 0x4831, { 0xa8, 0xe8, 0xa3, 0x95, 0xeb, 0x1f, 0xb8, 0x71 } };
class I2DEditor_KC : public IUnknown
{
public:
	ICALLTYPE Activate(VARIANT_BOOL activate) = 0;
	ICALLTYPE Init() = 0;
	ICALLTYPE AttachObject(ISection* iEditableObject) = 0;
	ICALLTYPE GetObject(ISection** iEditableObject) = 0;
	ICALLTYPE BuildPrizm(IStrip *strip, IPrizmInfinite_KC** prizm) = 0;
	IUNKNOWN_
};

// {DA410FA5-F6B2-4e91-8345-450337412B16}
static const GUID IID_I2DView2_KC = { 0xda410fa5, 0xf6b2, 0x4e91, { 0x83, 0x45, 0x45, 0x3, 0x37, 0x41, 0x2b, 0x16 } };
class I2DView2_KC : public IUnknown
{
public:
	ICALLTYPE CreateTemplate(I2DView_KC **templ) = 0;
	ICALLTYPE Align(I2DView_KC *templ, int type) = 0;
	ICALLTYPE SetupAlignment(I2DView_KC *target, int type) = 0;
	IUNKNOWN_
};

// {4604C8C2-B63A-4b71-8800-E42D40E550E5}
static const GUID IID_IError_KC = { 0x4604c8c2, 0xb63a, 0x4b71, { 0x88, 0x0, 0xe4, 0x2d, 0x40, 0xe5, 0x50, 0xe5 } };
class IError_KC : public IUnknown
{
public:
	ICALLTYPE GetError(BSTR *err) = 0;
	ICALLTYPE GetErrorEx(int errorNo, BSTR *err) = 0;
	ICALLTYPE GetCount(int *count) = 0;
	ICALLTYPE PushError(char* szMess, bool bWarning, bool bClear) = 0;	//Unpublished
	IUNKNOWN_
};

// {089378C3-9324-45db-9589-AB59C5A350C2}
static const GUID IID_IObjectPosition_KC = { 0x89378c3, 0x9324, 0x45db, { 0x95, 0x89, 0xab, 0x59, 0xc5, 0xa3, 0x50, 0xc2 } };
class IObjectPosition_KC : public IUnknown
{
public:
	ICALLTYPE FindStableTangentialPlane(DIPoint* pt0, DIPoint* pt1, DIPoint* pt2, double distMax, int* result) = 0;
	ICALLTYPE GetMinimumOrientedBoundingBox(IBox** box) = 0;
	ICALLTYPE AlignGeometryToFrame(IFrame* iFrame) = 0;
	IUNKNOWN_
};

// Set of objects with attached moves. The interface wraps their simultaneous positioning .
// Query IArray to access objects. Query IKCStack to save / restore position after moving 
// separate objects
class IKinematicSet_KCT : public IUnknown
{
public:
	ICALLTYPE AddObject(IMetricsT* object, IMove_KC* move, VARIANT_BOOL includeChildren) = 0;
	ICALLTYPE Reset() = 0;
	ICALLTYPE SetPosition(double t) = 0;
	IUNKNOWN_
};

// {0210EA1B-5503-4dc9-8EB0-12A40B853492}
static const GUID IID_IDiffSurface_KC = { 0x210ea1b, 0x5503, 0x4dc9, { 0x8e, 0xb0, 0x12, 0xa4, 0xb, 0x85, 0x34, 0x92 } };
class IDiffSurface_KC : public IUnknown
{
public:
	ICALLTYPE GetNormalAt(DIPoint* pointAt, DIVect* normal, int *count) = 0;
	ICALLTYPE GetProximity(DIPoint* pointAt, IVertex** vertex) = 0;
	ICALLTYPE GetLocalParametrization(DIPoint* pointAt, IKO_Geom_BSplineSurface** patch) = 0;
	IUNKNOWN_
};

// {32F75533-0638-4eee-9E1F-D5C01BB355E2}
static const GUID IID_IPrint_KC = { 0x32f75533, 0x638, 0x4eee, { 0x9e, 0x1f, 0xd5, 0xc0, 0x1b, 0xb3, 0x55, 0xe2 } };
class IPrint_KC : public IUnknown
{
public:
	ICALLTYPE PrintView(EPageOrientation orientation) = 0;
	ICALLTYPE GetDefaults(PrintContext_KC* options) = 0;
	ICALLTYPE Print(PrintContext_KC* options) = 0;
	ICALLTYPE PrintToFile(char* szFilePath, int width, int height, PrintContext_KC* options) = 0;
	ICALLTYPE Get2DView(I2DView_KC **view) = 0;
	ICALLTYPE StartDocument() = 0;
	ICALLTYPE EndDocument() = 0;
	IUNKNOWN_
};

// {D3CA1888-2BDA-4ec8-A865-E2548865B341}
static const GUID IID_IRectangleColor_KC = { 0xd3ca1888, 0x2bda, 0x4ec8, { 0xa8, 0x65, 0xe2, 0x54, 0x88, 0x65, 0xb3, 0x41 } };
class IRectangleColor_KC : public IUnknown
{
public:
	ICALLTYPE SetStockStyle(int style, Color_KC* color0, Color_KC* color1) = 0;
	ICALLTYPE SetColorCount(int side, int count) = 0;
	ICALLTYPE GetColorCount(int side, int* count) = 0;
	ICALLTYPE SetColor(int side, int index, float location, Color_KC* color) = 0;
	ICALLTYPE GetColor(int side, int index, float* location, Color_KC* color) = 0;
	IUNKNOWN_
};

// {DE6F0F95-67D5-4729-A448-6913DFF60531}
static const GUID IID_IMenuItem_KC = { 0xde6f0f95, 0x67d5, 0x4729, { 0xa4, 0x48, 0x69, 0x13, 0xdf, 0xf6, 0x5, 0x31 } };
class IMenuItem_KC : public IUnknown
{
public:
	ICALLTYPE GetText(BSTR *text) = 0;
	ICALLTYPE SetText(char* text) = 0;
	ICALLTYPE IsEnabled(VARIANT_BOOL *enabled) = 0;
	ICALLTYPE Enable(VARIANT_BOOL enable) = 0;
	ICALLTYPE GetIndex(int *index) = 0;
	ICALLTYPE SetIndex(int index) = 0;
	ICALLTYPE IsChecked(VARIANT_BOOL *checked) = 0;
	ICALLTYPE SetChecked(VARIANT_BOOL checked) = 0;
	ICALLTYPE IsSeparator(VARIANT_BOOL *separator) = 0;
	ICALLTYPE SetSeparator(VARIANT_BOOL separator) = 0;
	ICALLTYPE IsSubMenu(VARIANT_BOOL *submenu) = 0;
	ICALLTYPE GetPrevLevel(IMenuItem_KC** parent) = 0;
	ICALLTYPE Execute() = 0;
	IUNKNOWN_
};

// {EBB150BB-CE3C-40a2-B256-4B0C4A939000}
static const GUID IID_IMenu_KC = { 0xebb150bb, 0xce3c, 0x40a2, { 0xb2, 0x56, 0x4b, 0xc, 0x4a, 0x93, 0x90, 0x0 } };
class IMenu_KC : public IUnknown
{
public:
	ICALLTYPE Enable(int item, VARIANT_BOOL enable) = 0;
	ICALLTYPE GetItemCount(int *count) = 0;
	ICALLTYPE GetItemAt(int index, IMenuItem_KC **item) = 0;
	ICALLTYPE InsertAt(int indexAt, IMenuItem_KC **item) = 0;
	ICALLTYPE RemoveAt(int index) = 0;
	ICALLTYPE Add(IMenuItem_KC **item) = 0;
	ICALLTYPE RemoveAll() = 0;
	ICALLTYPE FindItem(char* name, int *index) = 0;
	ICALLTYPE InsertSubMenu(int indexAt, IMenu_KC** newSubMenu) = 0;
	ICALLTYPE GetSubMenu(int i, IMenu_KC** subMenu) = 0;
	IUNKNOWN_
};

// {6BE34BE0-E84A-4e08-8174-A252B1712EEF}
static const GUID IID_IKinematicSection_KC = { 0x6be34be0, 0xe84a, 0x4e08, { 0x81, 0x74, 0xa2, 0x52, 0xb1, 0x71, 0x2e, 0xef } };
class IKinematicSection_KC : public IUnknown
{
public:
	ICALLTYPE CreateSweptVolume(int resolution, ISection **section) = 0;
	ICALLTYPE Move(double from, double to) = 0;
	ICALLTYPE SetMovement(IMove_KC* iMove) = 0;
	ICALLTYPE GetMovement(IMove_KC** iMove) = 0;
	ICALLTYPE MoveLocal(double from, double to) = 0;
	IUNKNOWN_
};

// {B44D40F1-730C-4033-AF5D-977AF5C4748C}
static const GUID IID_ICutEffect_KC = { 0xb44d40f1, 0x730c, 0x4033, { 0xaf, 0x5d, 0x97, 0x7a, 0xf5, 0xc4, 0x74, 0x8c } };
class ICutEffect_KC : public IUnknown
{
public:
	ICALLTYPE Setup(ISection* object) = 0;
	ICALLTYPE Begin(IMove_KC* path) = 0;
	ICALLTYPE Show(float t) = 0;
	ICALLTYPE End() = 0;
	ICALLTYPE Chain(ICutEffect_KC* effectNextMove) = 0;
	IUNKNOWN_
};

// {DE086363-95FB-415C-A64A-7E9859EDE460}
static const GUID IID_ISpinEffect_KC = { 0xde086363, 0x95fb, 0x415c, { 0xa6, 0x4a, 0x7e, 0x98, 0x59, 0xed, 0xe4, 0x60 } };
class ISpinEffect_KC : public IUnknown
{
public:
	ICALLTYPE Setup(ISection* objectSpan, IView* iView) = 0;
	ICALLTYPE Begin(int durationMilliSeconds) = 0;
	ICALLTYPE End(int durationMilliSeconds) = 0;
	IUNKNOWN_
};

// {DD3E8647-4E4F-4472-9E8F-984353902597}
static const GUID IID_IRectangleAxisAligned_KC = { 0xdd3e8647, 0x4e4f, 0x4472, { 0x9e, 0x8f, 0x98, 0x43, 0x53, 0x90, 0x25, 0x97 } };
class IRectangleAxisAligned_KC : public IUnknown
{
public:
	ICALLTYPE GetRange(int axis, double* min, double* max) = 0;
	ICALLTYPE SetRange(int axis, double min, double max) = 0;
	IUNKNOWN_
};

// {69E0904C-8852-4820-82F6-B69AD69C0C53}
static const GUID IID_IMeasurement_KC = { 0x69e0904c, 0x8852, 0x4820, { 0x82, 0xf6, 0xb6, 0x9a, 0xd6, 0x9c, 0xc, 0x53 } };
class IMeasurement_KC : public IUnknown
{
public:
	ICALLTYPE GetPointCount(int* count) = 0;
	ICALLTYPE SetPointCount(int count) = 0;
	ICALLTYPE GetPoint(int i, double* x, double* y, double* z) = 0;
	ICALLTYPE SetPoint(int i, double x, double y, double z) = 0;
	ICALLTYPE GetText(BSTR *text) = 0;
	ICALLTYPE SetText(char* text) = 0;
	ICALLTYPE GetDirection(double* vx, double* vy, double* vz) = 0;
	ICALLTYPE SetDirection(double vx, double vy, double vz) = 0;
	ICALLTYPE GetTextLocation(double* dist) = 0;
	ICALLTYPE SetTextLocation(double dist) = 0;
	ICALLTYPE GetTextPosition(double* ratio) = 0;
	ICALLTYPE SetTextPosition(double ratio) = 0;
	IUNKNOWN_
};

// {BE0B32E0-6752-4624-AC9F-1A17774E1EE8}
static const GUID IID_IModelHealer_KC = { 0xbe0b32e0, 0x6752, 0x4624, { 0xac, 0x9f, 0x1a, 0x17, 0x77, 0x4e, 0x1e, 0xe8 } };
class IModelHealer_KC : public IUnknown
{
public:
	ICALLTYPE AssembleSolids() = 0;
	IUNKNOWN_
};

// {DEADE2C0-18BF-4589-BAA9-805EBC715285}
static const GUID IID_IEuclideanShortestPath_KC = { 0xdeade2c0, 0x18bf, 0x4589, { 0xba, 0xa9, 0x80, 0x5e, 0xbc, 0x71, 0x52, 0x85 } };
class IEuclideanShortestPath_KC : public IUnknown
{
public:
	ICALLTYPE AddObject(ISection* obj) = 0;
	ICALLTYPE AddAllObjects(IModel* objects) = 0;
	ICALLTYPE SetInputPointCount(int points) = 0;
	ICALLTYPE GetInputPointCount(int* points) = 0;
	ICALLTYPE SetInputPoint(int i, double x, double y, double z) = 0;
	ICALLTYPE GetInputPoint(int i, double* x, double* y, double* z) = 0;
	ICALLTYPE AddInputPoint(double x, double y, double z) = 0;
	ICALLTYPE GetInputPointArray(IArray3D** iArray) = 0;
	ICALLTYPE GetInputPath(ICurve** path) = 0;
	ICALLTYPE MakeInputValid() = 0;
	ICALLTYPE Calculate(VARIANT_BOOL checkValidInput) = 0;
	ICALLTYPE GetOutputPath(ICurve** path) = 0;
	ICALLTYPE Reset(VARIANT_BOOL pathOnly) = 0;
	ICALLTYPE GetPrecision(double* precision) = 0;
	ICALLTYPE SetPrecision(double precision) = 0;
	ICALLTYPE GetOutputPath2(ISurfacePointArrayKC** path) = 0;
	ICALLTYPE Calculate2(DIPoint* from, DIPoint* to, ISurfacePointArrayKC** path) = 0;
	ICALLTYPE CalculatePerSegment(ISurfacePointArrayKC** path) = 0;
	ICALLTYPE GetObjects(IArray** objects) = 0;
	ICALLTYPE CloneNoPaths(IEuclideanShortestPath_KC** esp) = 0;
	IUNKNOWN_
};

// {4295847D-D4EC-4A39-94C7-69723BFDC93B}
static const GUID IID_ISticky_KC = { 0x4295847d, 0xd4ec, 0x4a39, { 0x94, 0xc7, 0x69, 0x72, 0x3b, 0xfd, 0xc9, 0x3b } };
class ISticky_KC : public IUnknown
{
public:
	ICALLTYPE StickTo(ISection* surface) = 0;
	ICALLTYPE Detach() = 0;
	ICALLTYPE IsAttached(VARIANT_BOOL* state) = 0;
	IUNKNOWN_
};

// {371A43E1-5966-430D-ADB7-1AE87C994261}
static const GUID IID_ITextSection_KC = { 0x371a43e1, 0x5966, 0x430d, { 0xad, 0xb7, 0x1a, 0xe8, 0x7c, 0x99, 0x42, 0x61 } };
class ITextSection_KC : public IUnknown
{
public:
	ICALLTYPE SetText(char* text) = 0;
	ICALLTYPE GetText(BSTR* text) = 0;
	ICALLTYPE CreateGeometryFlat(int type, double heightGeometric, double spacing, char* fontName, IView* view) = 0;
	ICALLTYPE CreateGeometrySolid(double heightGeometric, double thickness, double spacing, char* fontName, IView* view) = 0;
	ICALLTYPE GetType(int* type) = 0;
	ICALLTYPE GetFontName(BSTR* name) = 0;
	ICALLTYPE GetGeometricWidth(double* width) = 0;
	IUNKNOWN_
};

// {52B2E8AB-F1F2-4C0A-A6C1-E340BFA46702}
static const GUID IID_IImageOriented_KC = { 0x52b2e8ab, 0xf1f2, 0x4c0a, { 0xa6, 0xc1, 0xe3, 0x40, 0xbf, 0xa4, 0x67, 0x2 } };
class IImageOriented_KC : public IUnknown
{
public:
	ICALLTYPE SetGeometricWidth(double width, VARIANT_BOOL keepRatio) = 0;
	ICALLTYPE GetGeometricWidth(double* width) = 0;
	ICALLTYPE SetGeometricHeight(double height, VARIANT_BOOL keepRatio) = 0;
	ICALLTYPE GetGeometricHeight(double* height) = 0;
	ICALLTYPE LoadImage(char* imageFilePath) = 0;
	ICALLTYPE SaveImage(char* imageFilePath) = 0;
	IUNKNOWN_
};

// {77294CCC-73A1-4FC1-9D70-030B1A73BE73}
static const GUID IID_ILabelSection_KC = { 0x77294ccc, 0x73a1, 0x4fc1, { 0x9d, 0x70, 0x3, 0xb, 0x1a, 0x73, 0xbe, 0x73 } };
class ILabelSection_KC : public IUnknown
{
public:
	ICALLTYPE GetText(IText** text) = 0;
	ICALLTYPE GetFont(IDIFont** font) = 0;
	ICALLTYPE SetGeometricHeight(double height) = 0;
	ICALLTYPE GetGeometricSize(double* width, double* height) = 0;
	IUNKNOWN_
};

// {D2924BD1-9680-41B9-8572-9A0DCE0F986D}
static const GUID IID_IControlInitialisationContext_KC = { 0xd2924bd1, 0x9680, 0x41b9, { 0x85, 0x72, 0x9a, 0xd, 0xce, 0xf, 0x98, 0x6d } };
class IControlInitialisationContext_KC : public IUnknown
{
public:
	ICALLTYPE Activate(VARIANT_BOOL active) = 0;
	ICALLTYPE IsActive(VARIANT_BOOL* active) = 0;
	ICALLTYPE ReleaseContext() = 0;
	ICALLTYPE SetModelPath(char* path) = 0;
	ICALLTYPE GetModelPath(BSTR* path) = 0;
	ICALLTYPE SetShowGlobalAxes(VARIANT_BOOL show) = 0;
	ICALLTYPE GetShowGlobalAxes(VARIANT_BOOL* show) = 0;
	ICALLTYPE SetBackgroundColor(float red, float green, float blue, float alpha) = 0;
	ICALLTYPE GetBackgroundColor(float* red, float* green, float* blue, float* alpha) = 0;
	ICALLTYPE SetRenderSolid(VARIANT_BOOL solid) = 0;
	ICALLTYPE GetRenderSolid(VARIANT_BOOL* solid) = 0;
	ICALLTYPE SetTransparBackgrEnabled(VARIANT_BOOL enabled) = 0;
	ICALLTYPE GetTransparBackgrEnabled(VARIANT_BOOL* enabled) = 0;
	ICALLTYPE SetVisible(VARIANT_BOOL visible) = 0;
	ICALLTYPE IsVisible(VARIANT_BOOL* visible) = 0;
	ICALLTYPE SetHandleComposited(VARIANT_BOOL handle) = 0;
	ICALLTYPE GetHandleComposited(VARIANT_BOOL* handle) = 0;
	ICALLTYPE ResetToDefaults() = 0;
	ICALLTYPE GetExtendedProperties(IKCContext** context) = 0;
	ICALLTYPE GetUserDefinedProperties(IKCContext** context) = 0;
	IUNKNOWN_
};

class ITechnology_DG;
class IDictionary_DG;
// {E076EF0D-AE5B-4B15-9EBD-C7AEADC43EC4}
static const GUID IID_IGlobalAPI_KC = { 0xe076ef0d, 0xae5b, 0x4b15, { 0x9e, 0xbd, 0xc7, 0xae, 0xad, 0xc4, 0x3e, 0xc4 } };
class IGlobalAPI_KC : public IUnknown
{
public:
	ICALLTYPE LockInitialisationContext(IControlInitialisationContext_KC** context) = 0;
	ICALLTYPE LaunchExplorer(IModel* model, IView* view) = 0;
	ICALLTYPE GetReferenceCount(IUnknown* anInterface, int* count) = 0;
	ICALLTYPE AddRefUnsafe(IUnknown* anInterface, int* count) = 0;
	ICALLTYPE ReleaseUnsafe(IUnknown* anInterface, int* count) = 0;
	ICALLTYPE GetTechnology(char* name, ITechnology_DG** technology) = 0;
	ICALLTYPE GetCoreContext(IDictionary_DG** context) = 0;
	IUNKNOWN_
};

// {D43BECA1-AA49-4970-8DA4-A5E0907D0594}
static const GUID IID_IDynamicScreenItem_KC = { 0xd43beca1, 0xaa49, 0x4970, { 0x8d, 0xa4, 0xa5, 0xe0, 0x90, 0x7d, 0x5, 0x94 } };
class IDynamicScreenItem_KC : public IUnknown
{
public:
	ICALLTYPE GetSize(double* sx, double* sy) = 0;
	ICALLTYPE SetSize(double sx, double sy) = 0;
	ICALLTYPE GetAttachmentPoint(double* x, double* y, double* z) = 0;
	ICALLTYPE SetAttachmentPoint(double x, double y, double z) = 0;
	IUNKNOWN_
};
// {054E0B01-1EAE-4e88-8353-73EA8EE56483}
static const GUID IID_ILabel3D_KC = { 0x54e0b01, 0x1eae, 0x4e88, { 0x83, 0x53, 0x73, 0xea, 0x8e, 0xe5, 0x64, 0x83 } };
class ILabel3D_KC : public IUnknown
{
public:
	ICALLTYPE GetText(IText** iText) = 0;
	ICALLTYPE GetFont(IDIFont** iFont) = 0;
	ICALLTYPE ShowBox(VARIANT_BOOL show) = 0;
	ICALLTYPE GetShowBox(VARIANT_BOOL* show) = 0;
	ICALLTYPE ShowPointer(VARIANT_BOOL show) = 0;
	ICALLTYPE GetShowPointer(VARIANT_BOOL* show) = 0;
	ICALLTYPE ShowText(VARIANT_BOOL show) = 0;
	ICALLTYPE GetShowText(VARIANT_BOOL* show) = 0;
	ICALLTYPE GetBoxLineStyle(ILineStyle_KC** iStyle) = 0;
	ICALLTYPE GetOrientation(float* orientationAngleDegrees) = 0;
	ICALLTYPE SetOrientation(float orientationAngleDegrees) = 0;
	IUNKNOWN_
};
// {1E9E1661-AF9D-4165-BF4C-B79779C74947}
static const GUID IID_IDynamicScreenItemManager_KC = { 0x1e9e1661, 0xaf9d, 0x4165, { 0xbf, 0x4c, 0xb7, 0x97, 0x79, 0xc7, 0x49, 0x47 } };
class IDynamicScreenItemManager_KC : public IUnknown
{
public:
	ICALLTYPE Enable(VARIANT_BOOL enable) = 0;
	ICALLTYPE IsEnabled(VARIANT_BOOL* enabled) = 0;
	ICALLTYPE GetItemCount(int* count) = 0;
	ICALLTYPE GetItem(int index, IDynamicScreenItem_KC** item) = 0;
	ICALLTYPE SetSpacing(int pixels) = 0;
	ICALLTYPE GetSpacing(int* pixels) = 0;
	ICALLTYPE SetSpacingItemToObjects(int pixels) = 0;
	ICALLTYPE GetSpacingItemToObjects(int* pixels) = 0;
	ICALLTYPE RemoveItem(int item) = 0;
	ICALLTYPE RemoveAll() = 0;
	ICALLTYPE EnableOnTop(VARIANT_BOOL enable) = 0;
	ICALLTYPE IsEnabledOnTop(VARIANT_BOOL* enabled) = 0;
	ICALLTYPE EnableOutside(VARIANT_BOOL enable) = 0;
	ICALLTYPE IsEnabledOutside(VARIANT_BOOL* enabled) = 0;
	ICALLTYPE Update() = 0;
	ICALLTYPE EnableHiddenItemRemoval(VARIANT_BOOL enable) = 0;
	ICALLTYPE IsEnabledHiddenItemRemoval(VARIANT_BOOL* enabled) = 0;
	IUNKNOWN_
};
// {1FC92998-8F8E-4b71-AEAA-B315C515769E}
static const GUID IID_ISmartLabelManager_KC = { 0x1fc92998, 0x8f8e, 0x4b71, { 0xae, 0xaa, 0xb3, 0x15, 0xc5, 0x15, 0x76, 0x9e } };
class ISmartLabelManager_KC : public IUnknown
{
public:
	ICALLTYPE GetLabel(int index, ILabel3D_KC** label) = 0;
	ICALLTYPE CreateLabelsAutomatic() = 0;
	ICALLTYPE CreateObjectLabelAutomatic(ISection* object, VARIANT_BOOL includeChildren, VARIANT_BOOL childrenOnly) = 0;
	ICALLTYPE SetStringSource(EStringSource type) = 0;
	ICALLTYPE GetStringSource(EStringSource* type) = 0;
	ICALLTYPE SetTextToRectangleMargin(float textHeightToMarginRatio) = 0;
	ICALLTYPE GetTextToRectangleMargin(float* textHeightToMarginRatio) = 0;
	ICALLTYPE AddNewLabel(ISection* parent, ILabel3D_KC** label) = 0;
	ICALLTYPE GetLabelCreationTemplate(ILabel3D_KC** label) = 0;
	ICALLTYPE GetExtendedProperties(IKCContext** context) = 0;
	IUNKNOWN_
};

// {551D1905-1EE5-4945-B8F6-5BCD00129C3B}
static const GUID IID_IDualMode_KC = { 0x551d1905, 0x1ee5, 0x4945, { 0xb8, 0xf6, 0x5b, 0xcd, 0x0, 0x12, 0x9c, 0x3b } };
class IDualMode_KC : public IUnknown
{
public:
	ICALLTYPE AttachFrameWindow(__int64 handleAncestor) = 0;
	ICALLTYPE Update() = 0;
	IUNKNOWN_
};

#include "InterfKO.h"
#include "DgInterface.h"
#include "DgSmartPtr.h"

