// DGTypesEx.h
#pragma once
#include <string>
#include <cmath>

class ColorDg : public Color_DG
{
public:
	float& operator [](const int index) { if (index == 0) return red; if (index == 1) return green;	if (index == 2) return blue; return alpha; }
	void Get(float& r, float& g, float& b) { r = red; g = green; b = blue; }
	void Get(float& r, float& g, float& b, float& a) { r = red; g = green; b = blue; a = alpha; }
	void Set(float r, float g, float b) { red = r; green = g; blue = b; alpha = 1.0f; }
	void Set(float r, float g, float b, float a) { red = r; green = g; blue = b; alpha = a; }
	void Copy(const ColorDg& clr) { red = clr.red; green = clr.green; blue = clr.blue; alpha = clr.alpha; }
	ColorDg() { red = green = blue = 0; alpha = 1.0f; };
	ColorDg(float r, float g, float b) { red = r; green = g; blue = b; alpha = 1.0f; }
	ColorDg(float r, float g, float b, float a) { red = r; green = g; blue = b; alpha = a; }
	ColorDg(const ColorDg& clr) { Copy(clr); }
};

class PairIntDg : public PairInt_DG
{
public:
	PairIntDg()	{first = 0; second = 0;	}
	PairIntDg(int a, int b)	{first = a; second = b;	}
	PairIntDg(PairIntDg& pr) { first = pr.first; second = pr.second;}
	void Set(int a, int b) { first = a; second = b;	}
};

class Dg_2D : public DG2D
{
public:
	Dg_2D() { x[0] = x[1] = 0; };
	Dg_2D(double x, double y) { DG2D::x[0] = x; DG2D::x[1] = y; };
	double Dist(Dg_2D& pt) { return sqrt((x[0] - pt.x[0])*(x[0] - pt.x[0]) + (x[1] - pt.x[1])*(x[1] - pt.x[1])); };
	double DistMax(Dg_2D& pt) { double dx = std::fabs(x[0] - pt.x[0]);	double dy = std::fabs(x[1] - pt.x[1]); return std::fmax(dx, dy); }
	bool Is(double x, double y, double tolerance) { Dg_2D pt(x, y); return DistMax(pt) < tolerance; }
	bool IsSame(Dg_2D& pt, double tolerance) { return DistMax(pt) < tolerance; }
	void Set(double x, double y) { DG2D::x[0] = x; DG2D::x[1] = y; }
	double& operator [](const int index) { return x[index]; }
};

// A point can be identified with a vector from origin to the point. So, say, Norm() below is length of the correspondent vector
class PointDg : public Point_DG
{
public:
	PointDg& operator=(const PointDg& pt) { for (byte i = 0; i < 3; i++) x[i] = pt.x[i]; return *this; }
	PointDg& operator+=(const PointDg& pt) { for (byte i = 0; i < 3; i++) x[i] += pt.x[i]; return *this; }
	PointDg& operator-=(const PointDg& pt) { for (byte i = 0; i < 3; i++) x[i] -= pt.x[i]; return *this; }
	double& operator [](const int index) { return x[index]; }
	void Copy(const PointDg& pt) { for (byte i = 0; i < 3; i++) x[i] = pt.x[i];}
	double Norm() const { return sqrt(Product(*this)); }
	double Dist(const PointDg& pt) { PointDg ths(*this); ths -= pt; return ths.Norm(); }
	double DistMax(const PointDg& pt) const
	{
		double dx = std::fabs(x[0] - pt.x[0]);
		double dy = std::fabs(x[1] - pt.x[1]);
		double  d = std::fmax(dx, dy);
		double dz = std::fabs(x[2] - pt.x[2]);
		return std::fmax(dz, d);
	}
	double Product(const PointDg& pt) const { return x[0] * pt.x[0] + x[1] * pt.x[1] + x[2] * pt.x[2]; }
	bool IsNormal(const PointDg& pt) { return Product(pt) < gTol; }
	bool Is(double x, double y, double z, double tolerance) { PointDg pt(x, y, z); return Dist(pt) < tolerance; }
	bool IsSame(const PointDg& pt, double tolerance=gTol) { return DistMax(pt) < tolerance; }
	void SwapDirection() { x[0] = -x[0]; x[1] = -x[1]; x[2] = -x[2]; }
	void Get(double& x, double& y, double& z) {x = Point_DG::x[0]; y = Point_DG::x[1]; z = Point_DG::x[2];}
	void Set(double x, double y, double z) { Point_DG::x[0] = x; Point_DG::x[1] = y; Point_DG::x[2] = z; };
	PointDg() { x[0] = x[1] = x[2] = 0; };
	PointDg(double x, double y, double z) { Point_DG::x[0] = x; Point_DG::x[1] = y; Point_DG::x[2] = z; };
	PointDg(const PointDg& pt) { x[0] = pt.x[0]; x[1] = pt.x[1]; x[2] = pt.x[2]; }
};

class VectDg : public Vect_DG
{
public:
	VectDg& operator=(const VectDg& vec) { for (byte i = 0; i < 3; i++) v[i] = vec.v[i]; return *this; }
	VectDg& operator+=(const VectDg& vec) { for (byte i = 0; i < 3; i++) v[i] += vec.v[i]; return *this; }
	VectDg& operator-=(const VectDg& vec) { for (byte i = 0; i < 3; i++) v[i] -= vec.v[i]; return *this; }
	VectDg& operator*=(double m) { for (byte i = 0; i < 3; i++) v[i] *= m; return *this; }
	double& operator[](const int index) { return v[index]; }
	void Copy(const VectDg& vec) { for (byte i = 0; i < 3; i++) v[i] = vec.v[i];}
	// cross product
	friend VectDg operator ^(const VectDg& v0, const VectDg& v1)
	{
		return { v0.v[1] * v1.v[2] - v1.v[1] * v0.v[2], v1.v[0] * v0.v[2] - v0.v[0] * v1.v[2], v0.v[0] * v1.v[1] - v1.v[0] * v0.v[1] };
	}
	// scalar product
	friend double operator *(const VectDg& v0, const VectDg& v1) { return v0.v[0] * v1.v[0] + v0.v[1] * v1.v[1] + v0.v[2] * v1.v[2]; }
	// multiply
	friend VectDg operator *(const VectDg& v, double m) { return{ v.v[0] * m, v.v[1] * m, v.v[2] * m }; }

	void Add(const VectDg& vec) { for (byte i = 0; i < 3; i++) v[i] += vec.v[i];}
	VectDg CrossProduct(const VectDg& v0, const VectDg& v1)
	{
		return{ v0.v[1] * v1.v[2] - v1.v[1] * v0.v[2], v1.v[0] * v0.v[2] - v0.v[0] * v1.v[2], v0.v[0] * v1.v[1] - v1.v[0] * v0.v[1] };
	}

	double Angle(const VectDg& v) const
	{
		double fPr = Product(v);
		double fMNorm = v.Length();
		double fNorm = Length();

		if (fNorm < 1e-10 || fMNorm < 1e-10)
			throw std::exception("Angle(): One of vectors has zero length");

		fPr /= fMNorm;
		fPr /= fNorm;

		if (fPr >(1.0 - 1e-7) || fPr < (-1.0 + 1e-7))
		{
			VectDg ths(*this), m(v);
			ths.Normalize();
			m.Normalize();

			double area = (ths ^ m).Length();

			ASSERT(area < 0.1); // Otherwise should not come here

			double ang = asin(area);

			if (fPr < -1e-10)
				ang = dPi - ang;

			return ang;
		}
		else
		{
			return acos(fPr);
		}
	}
	double Length() const { return sqrt(Product(*this)); }
	double LengthMax() { double m = __max(fabs(v[0]), fabs(v[1])); return __max(m, fabs(v[2])); }
	double Dist(const VectDg& vec) { VectDg ths(*this); ths -= vec; return ths.Length(); }
	double DistMax(const PointDg& pt) const
	{
		double dx = std::fabs(v[0] - pt.x[0]);
		double dy = std::fabs(v[1] - pt.x[1]);
		double  d = std::fmax(dx, dy);
		double dz = std::fabs(v[2] - pt.x[2]);
		return std::fmax(dz, d);
	}
	double DistMax(const VectDg& vec) const
	{
		double dx = std::fabs(v[0] - vec.v[0]);
		double dy = std::fabs(v[1] - vec.v[1]);
		double d = std::fmax(dx, dy);
		double dz = std::fabs(v[2] - vec.v[2]);
		return std::fmax(dz, d);
	}
	double Product(const VectDg& vec) const { return v[0] * vec.v[0] + v[1] * vec.v[1] + v[2] * vec.v[2]; }
	bool IsNormal(const VectDg& v) { return abs(Product(v)) < gTol; }
	bool Is(double vx, double vy, double vz, double tolerance) { VectDg vec(vx, vy, vz); return Dist(vec) < tolerance; }
	bool IsZero() { return IsZero(gTol); }
	bool IsZero(double tolerance) { return Is(0,0,0, tolerance); }
	bool IsSame(const VectDg& v, double tolerance) const { return DistMax(v) < tolerance; }
	bool IsCollinear(VectDg v, double tolerance) {double ds = CrossProduct(*this, v).LengthMax(); return ds < tolerance;	}
	void SwapDirection() { v[0] = -v[0]; v[1] = -v[1]; v[2] = -v[2]; }
	void Normalize()
	{
		double dNorm = Length();
		if (dNorm <= gTol)
			return;
		for (int i = 0; i<3; i++)
			v[i] /= dNorm;
	}
	void Get(double& vx, double& vy, double& vz) { vx = v[0]; vy = v[1]; vz = v[2]; }
	void Set(double vx, double vy, double vz) { v[0] = vx; v[1] = vy; v[2] = vz; }
	bool IsOrtho(const VectDg& vec, double tolerance) const { return abs(Product(vec)) < tolerance; }
	bool MakeOrtho(const VectDg& w)			// Apply orthogonalization to this vector
	{
		if (IsZero())
			return false;
		double sp = Product(w);
		if (sp < gTol)
			return true;
		if (IsCollinear(w, gTol))
			return false;
		double ww = w.Product(w);

		CHN((ww > 0));

		double mu = -sp / ww;

		VectDg wLoc(w);
		wLoc *= mu;

		Add(wLoc);

		ASSERT(IsOrtho(w, 100 * gTol));
		return true;
	}
	VectDg GetAnyOrthoVector()
	{
		// Pick the nicest wrt global axes. Try the global X first (preference)
		VectDg v(1, 0, 0);

		int iBest = 0;
		double aMax = Angle(v);

		if (aMax < 0.25 * dPi)
		{
			// X axis is not good. Find another
			for (byte i = 1; i < 3; i++)
			{
				v.Set(0, 0, 0);
				v[i] = 1;
				double aCur = Angle(v);
				if (aCur > aMax)
				{
					aMax = aCur;
					iBest = i;
				}
			}
		}

		VectDg vOrtho;
		vOrtho[iBest] = 1.0;

		vOrtho.MakeOrtho(*this);

		vOrtho.Normalize();

		ASSERT(IsOrtho(vOrtho, gTol));

		return vOrtho;
	}
	VectDg() { v[0] = v[1] = v[2] = 0; };
	VectDg(double vx, double vy, double vz) { v[0] = vx; v[1] = vy; v[2] = vz; }
	VectDg(const VectDg& vec) { v[0] = vec.v[0]; v[1] = vec.v[1]; v[2] = vec.v[2]; }
	VectDg(PointDg& pt0, PointDg& pt1) { v[0] = pt1[0]- pt0[0]; v[1] = pt1[1] - pt0[1]; v[2] = pt1[2] - pt0[2];	}
};

class RangeDg : public Range_DG
{
public:
	RangeDg() { min = 1; max = 0; };		// Empty
	RangeDg(double a, double b) { min = a; max = b; }
	RangeDg(Range_DG& rg) { min = rg.min; max = rg.max; };
	void Set(double a, double b) { min = a; max = b; }
	void SetEmpty() { min = 1; max = 0; }
	void Copy(Range_DG& rg){ min = rg.min; max = rg.max; }
	double GetValAtRatio(double ratio) { return min + ratio*(max - min); }
	double GetRatio(double val) { double d = max - min; if (d < gTol) return 0; return (val - min) / d; }
	double GetLength() const { return max - min; }
	bool IsIn(double x) { return x > min - 1e10 && x < max + 1e10; }
	bool Is(double mn, double mx) { return std::fabs(min - mn) < 1e-8 && abs(max - mx) < 1e-8; }
	bool IsFinite() const { return GetLength() < 1e10; }
};

class BoxDg : public Box_DG
{
public:
	BoxDg() : BoxDg(0, 0, 0, 0, 0, 0) { }
	BoxDg(double minx, double maxx, double miny, double maxy, double minz, double maxz)
	{
		range[0].min = minx; range[0].max = maxx;
		range[1].min = miny; range[1].max = maxy;
		range[2].min = minz; range[2].max = maxz;
	}
	BoxDg(RangeDg rangeX, RangeDg rangeY, RangeDg rangeZ)
	{
		range[0].min = rangeX.min; range[0].max = rangeX.max;
		range[1].min = rangeY.min; range[1].max = rangeY.max;
		range[2].min = rangeZ.min; range[2].max = rangeZ.max;
	}
	double Range(int axis) { return range[axis].max - range[axis].min; }
	Range_DG& operator[](const int index) { return range[index]; }
	RangeDg GetRange(int axis)
	{
		return RangeDg(range[axis]);
	}
	PointDg GetPoint(PointDg& ratios) 
	{
		PointDg pt; 
		for (byte i = 0; i < 3; i++)
			pt.x[i] = RangeDg(range[i]).GetValAtRatio(ratios.x[i]);
		return pt;
	}
	PointDg GetRatios(PointDg& point)
	{
		PointDg ratios; 
		for (byte i = 0; i < 3; i++)
			ratios.x[i] = RangeDg(range[i]).GetRatio(point.x[i]);
		return ratios;
	}
	double GetMaxDimension()
	{
		double d = 0;
		for (byte i = 0; i < 3; i++)
		{
			RangeDg rg = GetRange(i);
			double dcur = rg.GetLength();
			if (dcur > d)
				d = dcur;
		}
		return d;
	}
};

// Straight 3D Line as origin and direction
class LineDg : public Line_DG
{
public:
	LineDg() : LineDg(0, 0, 0, 0, 0, 0) { }
	LineDg(double originX, double originY, double originZ, double dirX, double dirY, double dirZ) { Set(originX, originY, originZ, dirX, dirY, dirZ); }
	LineDg(const PointDg& or, const VectDg& dir)
	{
		origin = or;
		direction = dir;
	}
	LineDg(const LineDg& line) { Copy(line); }
	void Set(double originX, double originY, double originZ, double dirX, double dirY, double dirZ)
	{
		origin.x[0] = originX;
		origin.x[1] = originY;
		origin.x[2] = originZ;
		VectDg dir(dirX, dirY, dirZ);
		dir.Normalize();
		direction.v[0] = dir[0];
		direction.v[1] = dir[1];
		direction.v[2] = dir[2];
	}
	void Copy(const LineDg& line)
	{
		origin = line.origin;
		direction = line.direction;
	}
};

// Plane as origin and direction of the normal
class PlaneDg : public LineDg
{
public:
	PlaneDg() : PlaneDg(0, 0, 0, 0, 0, 0) { }
	PlaneDg(double originX, double originY, double originZ, double normalX, double normalY, double normalZ) : LineDg(originX, originY, originZ, normalX, normalY, normalZ) {}
	PlaneDg(PointDg origin, VectDg normal) : LineDg(origin, normal) {}
};

// Straight 3D segment between two points defined as LineDg with finite length
class SegmentDg : public LineDg
{
public:
	double length = .0;
public:
	SegmentDg() : LineDg(0, 0, 0, 0, 0, 0) { }
	SegmentDg(const PointDg& pt0, const PointDg& pt1) { Set(pt0, pt1); }
	SegmentDg(double X0, double Y0, double Z0, double X1, double Y1, double Z1)
	{
		PointDg pt0(X0, Y0, Z0), pt1(X1, Y1, Z1);
		Set(pt0, pt1);
	}
	void Set(const PointDg& pt0, const PointDg& pt1)
	{
		for (byte i = 0; i < 3; i++) { origin.x[i] = pt0.x[i]; direction.v[i] = pt1.x[i] - pt0.x[i]; }
		VectDg dir(direction.v[0], direction.v[1], direction.v[2]);
		length = dir.Length();
		if(length < 1e-10)
			throw std::exception("Zero length segment");
		dir *= 1./length;
		dir.Get(direction.v[0], direction.v[1], direction.v[2]);
	}
};

class RectangleDg : public Rectangle_DG
{
public:
	RectangleDg() { SetEmpty(); }
	RectangleDg(double minx, double maxx, double miny, double maxy){ range[0].min = minx; range[0].max = maxx; range[1].min = miny; range[1].max = maxy; }
	RectangleDg(RangeDg& rangeX, RangeDg& rangeY){ range[0].min = rangeX.min; range[0].max = rangeX.max; range[1].min = rangeY.min; range[1].max = rangeY.max; }
	void SetEmpty() { SetRange(0, 1, 0); SetRange(1, 1, 0); }
	RangeDg GetRange(int axis) { return RangeDg(range[axis]); }
	void SetRange(int axis, double a, double b) { range[axis].min = a; range[axis].max = b; }
	void Copy(RectangleDg& rc) { range[0].min = rc.range[0].min; range[0].max = rc.range[0].max; range[1].min = rc.range[1].min; range[1].max = rc.range[1].max; }
	bool IsIn(double x, double y) { return GetRange(0).IsIn(x) && GetRange(1).IsIn(x); }
	bool IsIn(PointDg pt) { return IsIn(pt[0], pt[1]); }
};

class RegistryDg
{
public:
	static std::string GetInstallationPath()
	{
		HKEY hKeyVer = nullptr;
		std::string szKeyNameBuff = "SOFTWARE\\DInsight\\Common\\DGKernel\\" + std::to_string(VERSION_MAJOR) + "." + std::to_string(VERSION_MINOR);
		LONG lRes = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKeyNameBuff.c_str(), 0, KEY_READ, &hKeyVer);
		if (lRes != ERROR_SUCCESS)
			return FALSE;

		char szInstallDir[256] = "";
		DWORD dwSize = 256;
		lRes = RegQueryValueEx(hKeyVer, "LastInstallPath", NULL, NULL, (LPBYTE)szInstallDir, &dwSize);
		if (lRes != ERROR_SUCCESS)
		{
			RegCloseKey(hKeyVer);
			return FALSE;
		}
		RegCloseKey(hKeyVer);

		return szInstallDir;
	}
};

class UtilDg
{
public:
	static IEntity_DGPtr CreateEntity(char* type, IObjectGenerator_DGPtr& gen)
	{
		IEntity_DGPtr iEntity;
		gen->Create("IEntity_DG", (IUnknown**)&iEntity);
		iEntity->SetGeometryType(type, nullptr);
		return iEntity;
	}

	static std::string GetGeometryType(IEntity_DGPtr& iEntity)
	{
		BSTR cType = nullptr;
		iEntity->GetGeometryType(&cType);
		CString strType(cType);
		SysFreeString(cType);
		return std::string(strType.GetBuffer());
	}

	static CString GetEntityName(IEntity_DGPtr& iEntity)
	{
		BSTR c = nullptr;
		iEntity->GetName(&c);
		return CString(c);
	}

	static std::string GetName(IEntity_DGPtr& iEntity)
	{
		BSTR c = nullptr;
		iEntity->GetName(&c);
		CString str(c);
		SysFreeString(c);
		return std::string(str.GetBuffer());
	}
};
