
// ModFilletDlg.cpp : implementation file
//

#include "pch.h"
#include "framework.h"
#include "modFillet.h"
#include "ModFilletDlg.h"
#include "afxdialogex.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// ModFilletDlg dialog



ModFilletDlg::ModFilletDlg(CWnd* pParent /*=nullptr*/)
	: CDialogEx(IDD_MODFILLET_DIALOG, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void ModFilletDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_KERNELCADCTRL1, m_dgk);
}

BEGIN_MESSAGE_MAP(ModFilletDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_FILLET, &ModFilletDlg::OnFillet)
END_MESSAGE_MAP()


// ModFilletDlg message handlers

BOOL ModFilletDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_iModel = m_dgk.GetModelDG();
	m_iView = m_dgk.GetViewDG();
	m_gen = m_iModel;
	m_iDraw = m_iModel;

	m_dgk.SetShowGlobalAxes(TRUE);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void ModFilletDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR ModFilletDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void ModFilletDlg::FitView()
{
    m_iView->Reset(true, false);
}

void ModFilletDlg::OnFillet()
{
    IBRepFillet_DGPtr MyBRepFillet;
    IEntity_DGPtr BoxEntity;
    IEntity_DGPtr CylinderEntity;
    IBRepShape_DGPtr MyBRepShape;
    IBRepShape_DGPtr MyFilletShape;
    IBooleanEntity_DGPtr MyBooleanEntity;

    // create objects
    //IStdShape_DGPtr MyStdShape = MyObjGen3D.Create("IStdShape_DG")
    IStdShape_DGPtr MyStdShape = m_iModel;
    m_gen->Create("IBRepFillet_DG", (IUnknown**)&MyBRepFillet);

    // create shapes cylinders as entities, get BReps from entities for boolean operations
    MyStdShape->Box(7, 8, 1.5, &BoxEntity);
    // NZ. Try: MyStdShape->Rectangle(7, 8, &BoxEntity);

    MyStdShape->Cylinder(3, 4, 10, &CylinderEntity);   // cylinder with different radii produces bad geometry

    // moving box
    IFrame_DGPtr iFrameBox;
    BoxEntity->GetLocation(&iFrameBox);
    iFrameBox->Translate1(0, 0, 4);

    // boolean operation with CylinderEntity as base
    MyBooleanEntity = CylinderEntity;
    MyBooleanEntity->Unite(BoxEntity);

    // get boolean sub shapes (edges)
    MyBRepShape = MyBooleanEntity;
    IShapeArray_DGPtr EdgeArray;
    MyBRepShape->GetSubShapes(eShTypeEdgeDG, &EdgeArray);

    double dblRadius = 0.5;

#ifdef _DEBUG
    bool bDump = false;
    I3DBugger_DGPtr iDbg;
    if (bDump)
        iDbg = m_iModel;
#endif // _DEBUG

    // most edges OK, some crash the program (or erase cylinder)
    bool blnTest = true;      // set to False for code that crashes
    //blnTest = false;

    // trying to fillet intersection
    if(blnTest)
    {
        // === EXECUTES PROPERLY ===
        MyBRepFillet->Init(MyBRepShape, 0);
        IBRepShape_DGPtr shape2, shape3;
        IBRepEdge_DGPtr edge2, edge3;
        EdgeArray->GetAt(2, &shape2);
        edge2 = shape2;
        EdgeArray->GetAt(3, &shape3);
        edge3 = shape3;

#ifdef _DEBUG
        if (iDbg)
        {
            iDbg->DumpObject(MyBooleanEntity.GetInterfacePtr(), (POSN)MyBooleanEntity.GetInterfacePtr(), VARIANT_TRUE, VARIANT_FALSE);
            iDbg->DumpObject(edge2.GetInterfacePtr(), (POSN)edge2.GetInterfacePtr(), VARIANT_TRUE, VARIANT_FALSE);
        }
#endif // _DEBUG

        MyBRepFillet->Add(dblRadius, edge2);    // OK
        MyBRepFillet->Add(dblRadius, edge3);    // OK
        MyBRepFillet->Build(&MyFilletShape);
    }
    else
    {
        // === CRASHES ===
        MyBRepFillet->Init(MyBRepShape, 0);
        IBRepShape_DGPtr shape1, shape2, shape3, shape4;
        IBRepEdge_DGPtr edge1, edge2, edge3, edge4;
        EdgeArray->GetAt(2, &shape2);
        edge2 = shape2;
        EdgeArray->GetAt(3, &shape3);
        edge3 = shape3;
        EdgeArray->GetAt(1, &shape1);
        edge1 = shape1;
        EdgeArray->GetAt(4, &shape4);
        edge4 = shape4;

        MyBRepFillet->Add(dblRadius, edge2);    // OK
        MyBRepFillet->Add(dblRadius, edge3);    // OK
        MyBRepFillet->Add(dblRadius, edge1);    // CRASH
        MyBRepFillet->Add(dblRadius, edge4);    // CRASH
        MyBRepFillet->Build(&MyFilletShape);
    }

    // add filleted shape to model for viewing
    IEntity_DGPtr entityFillet;
    m_iModel->AddBRepShape(MyFilletShape, &entityFillet);
    entityFillet->SetName("Solid");

    // update and fit view in form
    FitView();
}
